#--
# ucert
#
# A  Ruby library for enterprise user account certification / recertification / audit project
#
# Copyright (c)  CMBNY Risk Department
#++

# Change-log
#
#
# 10/24/2017  Implement Ucer::ClearPar class.
# 10/24/2017  Re-vamp CitidirectSecuritiesTracker to handle the new 'dat' report format
# 09/29/2017  Re-vamp CitidirectBETracker parser due to the service entitlement report format change
# 09/19/2017  Fix YstTracker file encoding to acccomodate the new report format;
#               Implement CmbrunNYTracker class print_position method.
# 09/12/2017  Implement Ucert::AdpPayrollTracker class.
# 09/10/2017  Re-implement Ucert::T24Tracker class, in order to support XML entitlement report format.
# 08/02/2017  Implement Ucert::SomTracker class.
# 04/19/2017  Implement Ucert::CvmTracker class which is a HO developed system under YST SSO.
# 04/18/2017  Add 'yst_id_2_dn' method under Ucert::YstTracker class.
# 02/01/2017  Implement the Ucert::BloombergTracker class.
# 01/23/2017  Fix the bug in the 'parse_sy_uer_role_task_permissions_detail_report' of Ucert::Sage100Tracker class,
# 01/23/2017  Fix the bug in 'parse_jpm_user_entitlement_report' method of Ucert::JpmAccessTracker class.
# 01/23/2017  Fix the bug in the 'cntl_code_2_property_flag?' method of Ucert::AdTracker class.
# 01/23/2017  Fix the bug in the 'is_ip?' method of Ucert::Utils module.
# 01/18/2017  Fix the 3 bugs in the Ucert::FisPrimeTracker class.
# 01/12/2017  Merge the unit test cases from Kainan(Aaron) Zhang.
# 09/12/2016  Implement 'go_dptm_list' method in the Ucert::GoTracker class.
# 08/17/2016  Bug fixes in GoTracker 'save!' and 'insert_dn' methods.
# 08/15/2016  Refine AdTracker 'search' algorithm; refine Madison535Tracker 'insert_dn' method.
# 08/12/2016  Improve the search algorithm in the GoTracker and AdTracker classes.
# 08/10/2016  Implement 'parse_madison535_user_access_report_2' method in the class Ucert::Madison535Tracker,
#               i.e. the 2nd control system user entitlement report from GO.
# 08/01/2016  Implement Ucert::Madison535Tracker class and the associated bin utility.
# 06/23/2016  Implement Ucert::YstTracker class and the associated bin utility (一事通).
# 06/14/2016  Implement Ucert::WmsTracker class and the associated bin utility (Internal WMS credit report system).
# 05/11/2016  Remove 'db_tbls' file into 'data' directory; Refactor parse_be_user_entitlement_report method of
#                Ucert::CitidirectBETracker class, in order to accomodate the new entitlement report data format.
# 04/21/2016  Refactor Ucert::GoTracker class by Chris.
# 04/18/2016  Implement Ucert::FrbTracker class and the associated bin utility (Federal Reserve Banks services).
# 04/11/2016  Refactor Ucert::GoTracker class by Chris.
# 04/01/2016  Refactor Ucert::Sage100Tracker class, add 'parse_sy_uer_role_task_permissions_detail_report' method in
#               order to read additional task permissions report.
# 03/29/2016  Improve 'ad_searches_by_text' under Ucert::AdTracker class.
# 03/21/2016  Finish to check User Map Dn Match for all systems.
# 03/21/2016  Re-factor Ucert::AdTracker class; implement local_ip_2_host method for reverse DNS lookup.
# 03/17/2016  Fix a bug in the Ucert::CitidirectSecuritiesTracker class.
# 03/15/2016  Re-factor Ucert::AdTracker class; depreciate the 'adfind' connector support.
# 03/14/2016  Add 'demo/idm_ad_reload.rb' script to automate the 'cyber_idm' AD tables update automatically in the
#               server side.
# 03/09/2016  Add 'db_tbls' file into 'data' directory, where its contains the list of matching db table names; the
#               file would be needed by the app to perform database update.
# 03/08/2016  Add 'ad_delta' AD record change tracking support in the 'AdTracker' class.
# 03/04/2016  Fix a bug in the T24Tracker class 'parse_t24_user_report' method, where additional
#               application or group  entitlements were omitted.
# 03/04/2016  Improve the 'adgrep' bin utility to catch and display multiple matches.
# 02/29/2016  Implement additional logic in the GoTracker.go_searches_by_text method, to handle
#               more flexible user search string input.
# 02/29/2016  Implement Ucert::AixTracker class and the associated bin utility; implement better DN record change
#               detection mechanism across the board including GoTracker class.
# 02/25/2016  Implement Ucert::CitrixSharefileTracker class and the associated bin utility.
# 02/24/2016  Implement Ucert::EquinixTracker class and the associated bin utility;
#               remove the obsolete AccountMap class.
# 02/24/2016  Update search function to Ucert::GoTracker.
# 02/24/2016  Add computer search function to Ucert::AdTracker.
# 02/23/2016  Bug fix of the inconsistancy with STB map file.
# 02/23/2016  Add search function to UCert::GoTracker class.
# 02/23/2016  Implement Ucert::StbTracker class and the associated bin utility.
# 02/19/2016  Fix map file dn data loss bug.
# 02/19/2016  Implement Ucert::CitidirectBETracker class and the associated bin utility.
# 02/18/2016  Implement function to Ucert::GoTracker to export go contact sheet.
# 02/17/2016  Implement Ucert::AudiologTracker class and the associated bin utility.
# 02/16/2016  Implement Ucert::MantisTracker class and the associated bin utility.
# 02/11/2016  Add membership information into the 'ad_dump' bin utility.
# 02/11/2016  Implement Ucert::CmbrunNYTracker class and the associated bin utility.
# 01/20/2016  Modify 'ad_update' utility to disable support for 'adfind' support and Domain ID logon;
#               going forward, a valid DN would be needed. (Better Ubuntu Linux platform deployment experience).
# 01/20/2016  Implement Ucert::Check21Tracker class and the associated bin utility; fix a bug in
#               Ucert::SwiftOnlineTracker.
# 01/14/2016  Implement Ucert::SwiftOnlineTracker class and the associated bin utility.
# 01/13/2016  Fix a bug in the Ucert::FisPrimeTracker.insert_dn method; implement Ucert::VpnTracker class
#               and associated bin utility.
# 01/12/2016  Add two structural functions to Ucert::GoTracker.
# 01/12/2016  Implement Ucert::Sage100Tracker class and the associated bin utility.
# 01/11/2016  Re-factor 'insert_dn' mechanism, so that it would automatically detect and update the 'DN'
#               field when it loads.
# 01/08/2016  Re-factor the Ucert::T24Tracker class, abstract out the 'insert_dn' method (finally).
# 01/08/2016  Implement Ucert::DbDirectTracker class and the associated bin utility.
# 01/07/2016  Implement Ucert::JpmAccessTracker class and the associated bin utility.
# 12/30/2015  Implement Ucert::CitidirectSecuritiesTracker class and the associated bin utility.
# 12/30/2015  Fix a bug in the T24Tracker.parse_t24_user_report function.
# 12/23/2015  Update several modifications for upload functions to different trackers by Chris
# 12/14/2015  Update "Ucert::AdTracker.update_ad_cache" method and add sanity check for LDAP ID input.
# 12/02/2015  Implement 'Ucert::AllianceSwiftTracker' class and associated 'swift_dump' utility; minor
#               adjustment to the 'save!' method across all tracker classes.
# 12/01/2015  Re-implement 'adump.rb' by using 'Ucert::AdTracker' class; change its name to 'adgrep';
#               fix a small bug in the 'Ucert::AdTracker.ad_search_by_text' method
# 11/30/2015  Modification for 'Ucert::T24Tracker.t24_2_ad_user' method
# 11/24/2015  Implement Ucert::FisEgiftsTracker class; add 'egifts_dump' utility; minor improvement to
#               'Ucert::T24Tracker.t24_2_ad_user' method
# 11/13/2015  Refactor 'ad_dump', 'ad_update' utilities; add ability to dump out 'computer' table
# 11/12/2015  Refactor Ucert::AdTracker class: a) add support to openldap as an AD connector,
#               b) add 'get_dn_attribute' method
# 11/02/2015  Add 'search_by_dn' method for the Ucert::T24Tracker and Ucert::FisPrimeTracker classes.
# 10/30/2015  Add Ucert::Utils module; add Ucert::FisPrimeTracker class.
# 10/23/2015  Add class 'acct_cntl_code' definition back to Ucert::AdTracker class
# 10/08/2015  Ucert::T24Tracker class bug fixes and performance enhancement
# 10/08/2015  Add small utiliies under bin directory, i.e. 'ad_dump', 'ad_update', 't24_dump'
# 10/08/2015  Add Ucert::GoTracker class by Chris Gui
# 10/06/2015  Add Ucert::T24Tracker class
# 10/01/2015  Add 'get_os_info', 'cntl_code_2_property_flag', 'print' methods for Ucert::AdTracker class.
# 09/30/2015  Small bug fix of method 'ad_searches_by_text' to return Array under Exception condition.
# 09/29/2015  Add 'ad_searches_by_text', 'get_cns' methods for Ucert::AdTracker class.
# 09/15/2015  Add 'get_email_address', 'get_department', 'get_cn' for Ucert::AdTracker class, credit to Chris

## Mile-stones

- March 2016, go live with 'cyber_idm' app in Debian 14.0 LTS.
- Oct 2015, department proof-of-concept demo acceptance.
- June 2015, design considerations and prototypes.

### Backward Incompatibilities

- List of features that are backward incompatible:


## Beta Release 1.x


## Development Release 0.x

#
#	06/15/2015	Implement the active directory tracker class.
