require "minitest/autorun"
require "ucert"

#overriding initialize method for loading test_files
class Tracker_Ad < Ucert::AdTracker

  attr_accessor :ad_person_records,:ad_computer_records

  def initialize (params ={})
    @verbose=params.fetch(:verbose, false)
    @ldapsearch_cache_person=File.dirname(__FILE__)+"/ad_testfiles/ldap_person_test.txt"
    @ldapsearch_cache_computer=File.dirname(__FILE__)+"/ad_testfiles/ldap_computer_test.txt"
    @hosts_cache=File.dirname(__FILE__)+"/ad_testfiles/hosts_test"
    @ad_delta_map=File.dirname(__FILE__)+"/ad_testfiles/ad_delta_test.txt"
    @ad_person_records = Hash.new
    @ad_computer_records = Hash.new
  end
end

class TestAdTracker < Minitest::Test
  def setup
    @ad_tracker = Ucert::AdTracker.new(:verbose=>false)
    @tracker_ad = Tracker_Ad.new(:verbose=>false) #for test use
  end

  def test_ldap_server_is_bind
    assert @ad_tracker.is_ldap_bind?
    @ad_tracker = Ucert::AdTracker.new(:ldap_port=>"388")
    refute @ad_tracker.is_ldap_bind?
  end

  def test_load_known_hosts
    File.open(File.dirname(__FILE__)+"/ad_testfiles/hosts_test", "w") do |f|
      f.write "# local hosts file created"
      f.write "\ntest1.ny.cmbchina.com\t192.168.0.10"
      f.write "\ntest2.ny.cmbchina.com\t192.168.0.11"
    end
    result_host=@ad_tracker.send(:load_known_hosts,File.dirname(__FILE__)+"/ad_testfiles/hosts_test")
    test_host={"test1.ny.cmbchina.com"=>"192.168.0.10", "test2.ny.cmbchina.com"=>"192.168.0.11"}
    assert_equal test_host, result_host
  end

  def test_parse_openldap_cache
    @tracker_ad.send(:parse_openldap_cache, "person")
    @tracker_ad.send(:parse_openldap_cache, "computer")
    test_person_record={"test1"=>["a:apple", "b:banana", "c:cucumber"], "test2"=>["d:dog", "e:elephant", "f:frog"], "test3"=>["g:greed", "h:hell", "i:innocent"]}
    test_computer_record={"test1"=>["a:apple", "b:banana", "c:cucumber"], "test2"=>["d:dog", "e:elephant", "f:frog"], "test3"=>["g:greed", "h:hell", "i:innocent"]}
    assert_equal test_person_record, @tracker_ad.ad_person_records
    assert_equal test_computer_record, @tracker_ad.ad_computer_records
  end

  def test_ad_search_by_text
    @tracker_ad.ad_person_records={"test1"=>["a:apple", "b:banana", "c:cucumber"], "test2"=>["d:a pple", "e:ba nana", "f:cucum ber"]}
    assert_equal "test2", @tracker_ad.send(:ad_search_by_text,"pple a")
    assert_equal "test2", @tracker_ad.send(:ad_search_by_text,"ba nana")
    assert_equal "test1", @tracker_ad.send(:ad_search_by_text,"apple")
    refute @tracker_ad.send(:ad_search_by_text, "a big apple")
    @tracker_ad.ad_computer_records={"test1"=>["a:adam", "b:bob", "c:chris"], "test2"=>["d:a dam", "e:b ob", "f:c hris"]}
    assert_equal "test1", @tracker_ad.send(:ad_search_by_text, "a adam", "computer")
    refute @tracker_ad.send(:ad_search_by_text, "leo", "computer")
  end

  def test_ad_searchs_by_text
    @tracker_ad.ad_person_records={"test1"=>["a:apple", "b:banana", "c:cucumber"], "test2"=>["d:pple a", "e:ba nana", "f:cucum ber"],"test3"=>["g:greed", "h:hell", "i:innocent"]}
    #the order should be ["best match","close match"]
    assert_equal ["test2", "test1"], @tracker_ad.send(:ad_searches_by_text,"a pple")
  end

  def test_get_dn_attribute
    @tracker_ad.ad_person_records={"test1"=>["a:apple", "b:banana", "c:cucumber"], "test2"=>["d:a pple", "e:ba nana", "f:cucum ber"]}
    assert_equal "banana", @tracker_ad.send(:get_dn_attribute,"test1","b")
  end

  def test_get_dn_attributes
    @tracker_ad.ad_person_records={"test1"=>["a:apple", "a:banana", "a:cucumber", "b:pear"]}
    assert_equal ["apple", "banana", "cucumber"], @tracker_ad.send(:get_dn_attributes, "person", "test1", "a")
  end

#method may update, test will fail for now.
  def test_get_dn_attribute_and_cntl_code_2_property_flag
    acct_cntl_code={"SCRIPT"=>1, "ACCOUNTDISABLE"=>2, "HOMEDIR_REQUIRED"=>8, "LOCKOUT"=>16, "PASSWD_NOTREQD"=>32, \
      "PASSWD_CANT_CHANGE"=>64, "ENCRYPTED_TEXT_PWD_ALLOWED"=>128, "TEMP_DUPLICATE_ACCOUNT"=>256, "NORMAL_ACCOUNT"=>512, \
      "INTERDOMAIN_TRUST_ACCOUNT"=>2048, "WORKSTATION_TRUST_ACCOUNT"=>4096, "SERVER_TRUST_ACCOUNT"=>8192, \
      "DONT_EXPIRE_PASSWORD"=>65536, "MNS_LOGON_ACCOUNT"=>131072, "SMARTCARD_REQUIRED"=>262144, \
      "TRUSTED_FOR_DELEGATION"=>524288, "NOT_DELEGATED"=>1048576, "USE_DES_KEY_ONLY"=>2097152, "DONT_REQ_PREAUTH"=>4194304, \
      "PASSWORD_EXPIRED"=>8388608, "TRUSTED_TO_AUTH_FOR_DELEGATION"=>16777216, "PARTIAL_SECRETS_ACCOUNT"=>67108864}
    test_array=Array.new
    invert_acct=acct_cntl_code.invert
    @ad_tracker.ad_person_records.each do |key, value|
      test_array.push(@ad_tracker.get_dn_attribute("person",key,"userAccountControl"))
    end
    test_array.uniq!
    test_array.each do |num|
    puts "-----------------------------"
    test_control=Array.new
      puts num
      binary_num=num.to_i.to_s(2)
      puts binary_num
      puts binary_num.reverse
        for pos in 0...binary_num.length
          if binary_num.reverse[pos]=="1"
            puts pos
            puts invert_acct[2**pos]
            test_control.push(invert_acct[2**pos])
          end
        end
      assert_equal test_control.sort, @ad_tracker.send(:cntl_code_2_property_flag, num).strip.split(" + ").sort
    end
  end

  def test_save_hosts
    @ad_tracker.send(:save_hosts, File.dirname(__FILE__)+"/ad_testfiles/hosts_test")
    line_count=1
    File.open(File.dirname(__FILE__)+"/ad_testfiles/hosts_test", 'r').each_line do |line|
      if line_count==1
        assert_match  (/\# local hosts file created by the .+ class .+ method at: .+/), line
      else
        dn=line.split("\t")[0].strip
        ip=line.split("\t")[1].strip
        unless dn.empty?
          assert_equal ip, @ad_tracker.known_hosts[dn]
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/ad_testfiles/hosts_test")
  end

#test will fail since one ip may have mutiple host_names, method will only return the first match
  def test_local_ip_2_host
=begin
    @ad_tracker.send(:save_hosts, File.dirname(__FILE__)+"/ad_testfiles/hosts_test")
    line_count=1
    File.open(File.dirname(__FILE__)+"/ad_testfiles/hosts_test", 'r').each_line do |line|
      if line_count>1
        dn=line.split("\t")[0].strip
        ip=line.split("\t")[1].strip
        unless dn.empty?
          assert_equal dn, @ad_tracker.send(:local_ip_2_host, ip)
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/ad_testfiles/hosts_test")
=end
  end

end
