require "minitest/autorun"
require "ucert"

class TestAixTracker < Minitest::Test

  def setup
   @aix_tracker = Ucert::AixTracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
     refute_nil @aix_tracker.aix_passwd_file_list
     #refute_nil @aix_tracker.aix_user_status_report
     refute_nil @aix_tracker.file_aix_user_map
     refute_nil @aix_tracker.aix_2_ad_user
     refute_nil @aix_tracker.aix_user_entitlement
  end

  def test_parse_aix_passwd_files_method
    @test_entitlement=@aix_tracker.send(:parse_aix_passwd_files, [File.dirname(__FILE__)+"/aix_testfiles/application1.txt",File.dirname(__FILE__)+"/aix_testfiles/application2.txt"])
    assert_equal 2, @test_entitlement["test"]["accesses"].size
    assert_equal "1", @test_entitlement["foobar"]["accesses"]["application2"]["id"]
  end

  #method index should be username not number, so the test will fail
  def test_dn_2_index_method
    #refute_nil @aix_tracker.send(:dn_2_index, "CN=nyadmin,OU=SystemManagement,DC=ny,DC=cmbchina,DC=com")
  end

  def test_insert_dn
    @aix_tracker.aix_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

  def test_print_user_entitlement
    assert_output(/./) {@aix_tracker.print_user}
  end

  def test_save_aix_user_map!
    @aix_tracker.send(:save!, File.dirname(__FILE__)+"/aix_testfiles/aix_user_map_test.txt")
    line_count=1
    File.open(File.dirname(__FILE__)+"/aix_testfiles/aix_user_map_test.txt", 'r').each_line do |line|
      if line_count==1
        assert_match  (/\# local AIX to AD user map file created by the .+ class .+ method at: .+/), line
      else
        user_name=line.split("|")[0]
        line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        if dn.empty?
          assert_nil @aix_tracker.aix_user_entitlement[user_name]["DN"]
        else
          assert_equal @aix_tracker.aix_user_entitlement[user_name]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/aix_testfiles/aix_user_map_test.txt")
  end

  def test_db_search_by_dn
    index=0
    File.open(@aix_tracker.file_aix_user_map, 'r').each_line do |line|
      if index>0
        line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @aix_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
      end
      index=index+1
    end
  end

end
