require "minitest/autorun"
require "ucert"

class SwiftTestTracker < Ucert::AllianceSwiftTracker
  attr_accessor  :verbose, :swift_operator_details_report, :swift_operator_profile_details_report, :file_operator_map
  attr_reader    :swift_operator_entitlement, :swift_operator_profiles, :swift_2_ad_operator
  # Instance default variables
  def initialize (params ={})
    @verbose=params.fetch(:verbose, false)
    @swift_operator_details_report = File.dirname(__FILE__)+"/alliance_swift_testfiles/Swift_Operator_Details_Test.xlsx"
    @swift_operator_profile_details_report =  File.dirname(__FILE__)+"/alliance_swift_testfiles/Swift_Operator_Profiles_Details_Test.xlsx"
    @file_operator_map =  File.dirname(__FILE__)+"/alliance_swift_testfiles/swift_operator_map_test.txt"
    @swift_2_ad_operator=load_known_user_map_from_file(@file_operator_map)
    @swift_operator_entitlement=Hash.new
    parse_swift_operator_details_report(@swift_operator_details_report)
    @swift_operator_profiles=Hash.new
    parse_swift_operator_profile_details_report(@swift_operator_profile_details_report)
    insert_dn
    save!
  end
end

class TestAllianceSwiftTracker < Minitest::Test

  FileUtils.cp_r(File.dirname(__FILE__)+"/../data/alliance_swift/swift_operator_map.txt", File.dirname(__FILE__)+"/alliance_swift_testfiles/swift_operator_map_test.txt")

  def setup
   @test_swift_tracker = SwiftTestTracker.new(:verbose=>false)
   @swift_tracker = Ucert::AllianceSwiftTracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
     refute_nil @swift_tracker.swift_operator_details_report
     refute_nil @swift_tracker.swift_operator_profile_details_report
     refute_nil @swift_tracker.file_operator_map
     refute_nil @swift_tracker.swift_2_ad_operator
     refute_nil @swift_tracker.swift_operator_entitlement
     refute_nil @swift_tracker.swift_operator_profiles
  end

  #the method will use private variable swift_operator_entitlement for self sanity quick checking so cannot use testfiles
  def test_parse_swift_operator_details_report
    user1=["BAISHUO780","Shuo Bai; F&A","Approved/Disabled","2014/12/15 10:41:25","Human","Password","","R7.0_SysOper","None"]
    user1.all? do |e|
      assert_includes  @test_swift_tracker.swift_operator_entitlement[1].values, e
    end
    user50=["egifts","eGiftes System User","Approved/Enabled","2016/11/24 02:17:09","Human","Password","","R62_SuperKeyPLUS","None"]
    user50.all? do |e|
        assert_includes  @test_swift_tracker.swift_operator_entitlement[54].values, e
    end
    assert_equal 54,  @test_swift_tracker.swift_operator_entitlement.size
  end

  def test_parse_swift_operator_profile_details_report
    operator1=Hash.new
    operator1['Name']="R6.2_CFO"
    operator1["Access Control"]=Hash.new
    operator1["Access Control"]={"Files On User Space"=>[], "Signon"=>["Permissions =", "Start time: 0000", "End time: 2357", "Start time: 2358", "End time: 2359", "WS Session Timeout: 0", ""]}
    assert_equal operator1['Name'],  @test_swift_tracker.swift_operator_profiles[1]['Name']
    assert_equal operator1["Access Control"],  @test_swift_tracker.swift_operator_profiles[1]["Entities"]["Access Control"]
    assert_equal 43,  @test_swift_tracker.swift_operator_profiles.size
  end

  def test_insert_dn
    @test_swift_tracker.swift_operator_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

  def test_operator_name_2_index
    line_cnt=1
    File.open(File.dirname(__FILE__)+"/alliance_swift_testfiles/swift_operator_map_test.txt", 'r').each_line do |line|
      if line_cnt>1
        name=line.split(":")[0]
        #  @test_swift_tracker.swift_operator_entitlement
        assert_equal line_cnt-1,  @swift_tracker.send(:operator_name_2_index, name)
      end
      line_cnt+=1
    end
  end

  def test_print_operator_entitlement
    assert_output(/./) { @test_swift_tracker.print_operators}
  end

  def test_print_operator_profiles
    assert_output(/./) { @test_swift_tracker.print_profiles}
  end

#test will fail for the map's format is not adjusted
  def test_save_swift_operator_map!
    @swift_tracker.send(:save!, File.dirname(__FILE__)+"/alliance_swift_testfiles/swift_operator_map_test_2.txt")
    #test_files=["swift_operator_map_test_format_fixed.txt","swift_operator_map_test.txt"]
    test_files=["swift_operator_map_test_2.txt"]
    test_files.each do |testfile|
      line_count=0
      puts "This is the test for "+testfile
      File.open(File.dirname(__FILE__)+"/alliance_swift_testfiles/"+testfile, 'r').each_line do |line|
        if line_count==0
          assert_match (/\# local Swift to AD operator map file created by the .+ class .+ method at: .+/), line
        else
          user_name=line.split(":")[0]
          description=line.split(":")[1].split("|")[0]
          dn=line.split(":")[1].split("|")[1].nil? ? dn="" : dn=line.split(":")[1].split("|")[1].strip
          assert_equal @swift_tracker.swift_operator_entitlement[line_count]["Name"].upcase, user_name
          assert_equal @swift_tracker.swift_operator_entitlement[line_count]["Description"].upcase.gsub(/\s+/, ' ').strip, description
          if dn.empty?
            assert_nil @swift_tracker.swift_operator_entitlement[line_count]["DN"]
          else
            #puts @test_swift_tracker.swift_operator_entitlement[line_count]
            assert_equal @swift_tracker.swift_operator_entitlement[line_count]["DN"], dn
          end
        end
        line_count+=1
      end
      puts "Passed"
    end
  end

  def test_swift_search_by_dn
    index=0
    File.open(File.dirname(__FILE__)+"/alliance_swift_testfiles/swift_operator_map_test.txt", 'r').each_line do |line|
      if index>0
        line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @swift_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
      end
      index=index+1
    end
  end
end
