require "minitest/autorun"
require "ucert"

class TestCitidirectBETracker < Minitest::Test

  def setup
    @citi_be_tracker = Ucert::CitidirectBETracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
     refute_nil @citi_be_tracker.file_be_user_map
     refute_nil @citi_be_tracker.be_2_ad_user
     refute_nil @citi_be_tracker.be_user_entitlement
     #puts @citi_be_tracker.be_user_entitlement
  end

#Two reasons for test failure:
#1:The "Assigned_Access_Profiles" field is not loaded correctly, the code for assign cur_profiles line:138 is not executed due to the 'next' code in
# include?("Assigned Access Profiles") line:95 breaks the loop, which also caused "Billing Account Number" inforamtion not assigned correctly.
#2:The format for "Credential Type" and "Credential ID" is different from previous column, the key and value are vertical but not horizonal.
  def test_parse_be_user_entitlement_report
    @test_citi_be_entitlement=@citi_be_tracker.send(:parse_be_user_entitlement_report, File.dirname(__FILE__)+"/citidirect_be_testfiles/UserProfileEntitlementsReport_Test.xlsx")
    assert_equal 3, @test_citi_be_entitlement.size
    @test_citi_be_entitlement.each do |index, user|
      user.each do |key, value|
        if key=="Assigned_Access_Profiles"
          #refute value.empty?
          value.each do |title, content|
            assert_equal "test#{index}-"+title, content
          end
        elsif value.include?(" ")
          key_trans=key.gsub('_',' ')
          assert_equal "test#{index}-"+key_trans, value
        elsif value.include?("/")
          key_trans=key.gsub('_','/')
          assert_equal "test#{index}-"+key_trans, value
        else
          assert_equal "test#{index}-"+key, value
        end
      end
    end
  end

  def test_dn_2_index
    line_cnt=1
    File.open(@citi_be_tracker.file_be_user_map, 'r').each_line do |line|
      if line_cnt>1
        dn=line.split("|")[1].strip
        assert_equal line_cnt-1, @citi_be_tracker.send(:dn_2_index, dn)
      end
      line_cnt+=1
    end
  end

  def test_insert_dn
    @citi_be_tracker.be_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

#The test will fail because some user's Building Floor inforamtion is blank, so in the entitlement the building floor filed is nil,
#then some user will have 19 fields but some only have 18.
  def test_print_user_entitlement
    column_num=Array.new
    @citi_be_tracker.be_user_entitlement.values.map do |rec|
      column_num.push(rec.each.size)
    end
    assert_equal 1, column_num.uniq!.size
    $stdout = StringIO.new
    @citi_be_tracker.print_user.each
    items=$stdout.string.split("\n")
    items.each do |line|
     assert_equal column_num[0], line.count("|")
    end
  end

  def test_save_be_user_map
    @citi_be_tracker.send(:save!, File.dirname(__FILE__)+"/citidirect_be_testfiles/be_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/citidirect_be_testfiles/be_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# local Citidirect BE to AD user map file created by the .+ class .+ method at: .+/), line
      else
        email=line.split("|")[0]
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal @citi_be_tracker.be_user_entitlement[line_count]["E-Mail_Address"].upcase, email
        if dn.empty?
          assert_nil @citi_be_tracker.be_user_entitlement[line_count]["DN"]
        else
          assert_equal @citi_be_tracker.be_user_entitlement[line_count]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/citidirect_be_testfiles/be_user_map_test.txt")
  end

  def test_be_search_by_dn
    index=0
    File.open(@citi_be_tracker.file_be_user_map, 'r').each_line do |line|
      if index>0
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @citi_be_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
      end
      index=index+1
    end
  end

end
