require "minitest/autorun"
require "ucert"

class TestCitidirectSecuritiesTracker < Minitest::Test

  def setup
    @citi_se_tracker = Ucert::CitidirectSecuritiesTracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
    #refute_nil @citi_se_tracker.cds_user_status_report
     refute_nil @citi_se_tracker.file_cds_user_map
     refute_nil @citi_se_tracker.cds_2_ad_user
     refute_nil @citi_se_tracker.cds_user_entitlement
  end

  def test_parse_cds_user_entitlement_report
    @test_cds_user_entitlement=@citi_se_tracker.send(:parse_cds_user_entitlement_report, File.dirname(__FILE__)+"/citidirect_securities_testfiles/User_Entitlements_Report___CLNT_Test.xml")
    assert_equal 5, @test_cds_user_entitlement.size
    @test_cds_user_entitlement.each do |index, user|
      assert_equal "Test#{index}-1", user['UserName']
      assert_equal "Test#{index}-2", user['Created']
      assert_equal "Test#{index}-3", user['LastLogin']
      assert_equal "Test#{index}-4", user['DataGroup']
      assert_equal "Test#{index}-5", user['FunctionalGroup']
      assert_equal "Test#{index}-6", user['PwdExpired']
      assert_equal "Test#{index}-7", user['AcctDisabled']
    end
  end

  def test_dn_2_index
    line_cnt=1
    File.open(@citi_se_tracker.file_cds_user_map, 'r').each_line do |line|
      if line_cnt>1
        dn=line.split("|")[1].strip
        assert_equal line_cnt-1, @citi_se_tracker.send(:dn_2_index, dn)
      end
      line_cnt+=1
    end
  end

  def test_insert_dn
    @citi_se_tracker.cds_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

  def test_print_user_entitlement
    $stdout = StringIO.new
    @citi_se_tracker.print
    items=$stdout.string.split("\n")
    items.each do |line|
      assert_equal 7, line.count("|")
    end
  end

  def test_save_cds_user_map!
    @citi_se_tracker.send(:save!, File.dirname(__FILE__)+"/citidirect_securities_testfiles/citidirect_securities_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/citidirect_securities_testfiles/citidirect_securities_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# local CitiDirect for Securitites to AD user map file created by the .+ class .+ method at: .+/), line
      else
        user_name=line.split("|")[0]
        dn=line.split("|")[1].strip
        assert_equal @citi_se_tracker.cds_user_entitlement[line_count]["UserName"].upcase,user_name
        if dn.empty?
          assert_nil @citi_se_tracker.cds_user_entitlement[line_count]["DN"]
        else
          assert_equal @citi_se_tracker.cds_user_entitlement[line_count]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/citidirect_securities_testfiles/citidirect_securities_user_map_test.txt")
  end

  def test_cds_search_by_dn
    index=0
    File.open(@citi_se_tracker.file_cds_user_map, 'r').each_line do |line|
    if index>0
      dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
      assert_equal dn, @citi_se_tracker.send(:search_by_dn, dn)["DN"] unless dn.empty?
    end
      index=index+1
    end
  end
end
