require "minitest/autorun"
require "ucert"

class TestCitrixSharefileTracker < Minitest::Test

  def setup
    @citrix_tracker = Ucert::CitrixSharefileTracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
     refute_nil @citrix_tracker.file_sharefile_user_map
     refute_nil @citrix_tracker.sharefile_2_ad_user
     refute_nil @citrix_tracker.sharefile_user_entitlement
  end

  def test_parse_sharefile_user_entitlement_report
    @test_entitlement=@citrix_tracker.send(:parse_sharefile_user_entitlement_report, File.dirname(__FILE__)+"/citrix_sharefile_testfiles/ShareFile_Access_Report_Test.xlsx")
    assert_equal 3, @test_entitlement.size
    #puts @test_entitlement
    index=1
    @test_entitlement.each do |email, content|
      assert_equal ("test#{index}-Email").upcase, email
      content.each do |key, value|
        if key=="entitlements"
          1.upto(index) do |num|
            value[num-1].each do |k,v|
              assert_equal "test#{index}-#{k}", v
            end
          end
        else
          assert_equal "test#{index}-#{key}", value
        end
      end
      index+=1
    end
  end

#Test will fail because the entitlement use email as index but not number in the tracker code
  def test_dn_2_index
=begin
    line_cnt=1
    File.open(@citrix_tracker.file_sharefile_user_map, 'r').each_line do |line|
      if line_cnt>1
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal line_cnt-1, @citrix_tracker.send(:dn_2_index, dn) unless dn.empty?
      end
      line_cnt+=1
    end
=end
  end

  def test_insert_dn
    @citrix_tracker.sharefile_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

  def test_print_user_entitlement
      column_num=Array.new
      @citrix_tracker.sharefile_user_entitlement.values.map do |rec|
        column_num.push(rec.each.size)
      end
      assert_equal 1, column_num.uniq!.size
      $stdout = StringIO.new
      @citrix_tracker.print_user.each
      items=$stdout.string.split("\n")
      items.each do |line|
        assert_equal column_num[0], line.count("|")
      end
  end

  def test_save_sharefile_user_map!
    @citrix_tracker.send(:save!, File.dirname(__FILE__)+"/citrix_sharefile_testfiles/sharefile_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/citrix_sharefile_testfiles/sharefile_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# ShareFile to AD user map file created by the .+ class .+ method at: .+/), line
      else
        email=line.split("|")[0]
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal @citrix_tracker.sharefile_user_entitlement[email]["Email"].upcase, email
        if dn.empty?
          assert_nil @citrix_tracker.sharefile_user_entitlement[email]["DN"]
        else
          assert_equal @citrix_tracker.sharefile_user_entitlement[email]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/citrix_sharefile_testfiles/sharefile_user_map_test.txt")
  end

  def test_search_by_dn
    index=0
    File.open(@citrix_tracker.file_sharefile_user_map, 'r').each_line do |line|
      if index>0
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @citrix_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
      end
      index=index+1
    end
  end

end
