require "minitest/autorun"
require "ucert"

class TestDbDirectTracker < Minitest::Test

  def setup
   @db_tracker = Ucert::DbDirectTracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
    refute_nil @db_tracker.file_db_user_map
    refute_nil @db_tracker.db_2_ad_user
    refute_nil @db_tracker.db_user_entitlement
    puts @db_tracker.db_user_entitlement[2]["Entitlements"]
  end

  # infromation in the first row is not recorded except first user, which causes failure.
  def test_parse_db_user_entitlement_report_method
    #prepare test_file, only uncomment it when test_file is empty
=begin
        doc = File.open(File.dirname(__FILE__)+"/db_direct_testfiles/accountpermission_Test.csv",'w')
        title="User Id,WebSSO Id,First Name,Last Name,Product,Company,Bank Branch,Account,Account Currency,User Role Type,User Role,\"Entitlement Permission (View, Edit, Verify(optional), Use)\",\"Authorization Permission(Yes, No)\",\"Signatory Permission(Yes, No)\",Authorization Permission-LimitCurrency,Authorization Permission-Single Normal Limit,Authorization Permission-Single Pre-Approved Limit,Authorization Permission-Joint Category"
        doc.write(title)
        doc.write("\n")
        for kk in 1..3 do
          for ii in 1..kk do
            for jj in 1..7 do
              doc.write("Test"+kk.to_s+":"+jj.to_s+",")
            end
            for jj in 8..18 do
              doc.write("Test"+kk.to_s+":"+ii.to_s+":"+jj.to_s+",")
            end
            doc.write("\n")
          end
        end
      doc.close
=end

    @test_entitlement=@db_tracker.send(:parse_db_user_entitlement_report, File.dirname(__FILE__)+"/db_direct_testfiles/accountpermission_Test.xlsx")
    assert_equal 3, @test_entitlement.size
    #count=1
    for kk in 1..3 do
      for ii in 1..kk do
        for jj in 1..7 do
          #puts count
          assert_equal "Test"+kk.to_s+":"+jj.to_s, @test_entitlement[kk].values[jj-1]
          #count+=1
        end
        for jj in 8..18 do
          #puts count
          assert_equal "Test"+kk.to_s+":"+ii.to_s+":"+jj.to_s, (@test_entitlement[kk].values[7][ii-1].values)[jj-8]
          #count+=1
        end
      end
    end
  end

  def test_dn_2_index
    line_cnt=1
    File.open(@db_tracker.file_db_user_map, 'r').each_line do |line|
      if line_cnt>1
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal line_cnt-1, @db_tracker.send(:dn_2_index, dn)
      end
      line_cnt+=1
    end
  end

  def test_insert_dn
    @db_tracker.db_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

 #field 'Account', 'Account Currency' are in Entitlement which cause the test fail
  def test_print_user_entitlement
      column_num=Array.new
      @db_tracker.db_user_entitlement.values.map do |rec|
        #puts rec
        #puts "-----------------------------------------------------------"
        column_num.push(rec.each.size)
      end
      assert_equal 1, column_num.uniq!.size
      $stdout = StringIO.new
      @db_tracker.print_user.each
      items=$stdout.string.split("\n")
      items.each do |line|
        assert_equal column_num[0]-1, line.count("|")
      end
  end

  def test_save_db_user_map!
    @db_tracker.send(:save!, File.dirname(__FILE__)+"/db_direct_testfiles/db_direct_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/db_direct_testfiles/db_direct_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# local db for Securitites to AD user map file created by the .+ class .+ method at: .+/), line
      else
        user_id=line.split("|")[0]
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal @db_tracker.db_user_entitlement[line_count]["User Id"].upcase, user_id
        if dn.empty?
          assert_nil @db_tracker.db_user_entitlement[line_count]["DN"]
        else
          assert_equal @db_tracker.db_user_entitlement[line_count]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/db_direct_testfiles/db_direct_user_map_test.txt")
  end

  def test_db_search_by_dn
    index=0
    File.open(@db_tracker.file_db_user_map, 'r').each_line do |line|
      if index>0
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @db_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
      end
      index=index+1
    end
  end

end
