require "minitest/autorun"
require "ucert"
require 'fileutils'

#overriding initialize method for loading test_files
class Tracker_3quinix < Ucert::EquinixTracker
  def initialize (params ={})
    @verbose=params.fetch(:verbose, false)
    @equinix_user_entitlement_report=File.dirname(__FILE__)+"/equinix_testfiles/Secured Access List_CHINA MERCHANTS BANK_TEST.xlsx"
    @file_equinix_user_map=File.dirname(__FILE__)+"/equinix_testfiles/equinix_user_map_test2.txt"
    @equinix_user_entitlement=Hash.new
    parse_equinix_user_entitlement_report(@equinix_user_entitlement_report)
  end
end

class TestEquinixTracker < Minitest::Test

    def setup
      @equinix_tracker = Ucert::EquinixTracker.new(:verbose=>false)
      @tracker_3quinix = Tracker_3quinix.new(:verbose=>false) #for test use
      #The test_file would be updated with Active Directory
    end

    def test_instance_initialize_success
      refute_nil @equinix_tracker.file_equinix_user_map
      refute_nil @equinix_tracker.equinix_2_ad_user
      refute_nil @equinix_tracker.equinix_user_entitlement
    end

    def test_parse_equinix_user_entitlement_report
      @tracker_3quinix.equinix_user_entitlement.each do |index, user|
        assert_equal 3, user["entitlements"].size
      end
    end

    def test_parse_equinix_worksheet
      @tracker_3quinix.equinix_user_entitlement.each do |index, user|
        assert_equal "test#{index}-"+"Username:", user["Username:"]
        assert_equal "test#{index}-"+"First Name:", user["First Name:"]
        assert_equal "test#{index}-"+"Last Name:", user["Last Name:"]
        assert_equal "test#{index}-"+"Name (Local)", user["Name (Local)"]
        for sheet_count in 1..3 do
          for i in 1..sheet_count do
            assert_equal "test#{index}-"+"sheet"+sheet_count.to_s+"-"+i.to_s, user["entitlements"][sheet_count-1]["sheet"+sheet_count.to_s+"-"+i.to_s]
          end
        end
      end
    end

  def test_insert_dn
    @equinix_tracker.equinix_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

# In ecured Access List_CHINA MERCHANTS BANK .xlsx Douglas has two rows in portal access sheet, which caused the fail
  def test_dn_2_index
=begin
    index=1
    File.open(@equinix_tracker.file_equinix_user_map, 'r').each_line do |line|
      if index>1
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal index-1, @equinix_tracker.send(:dn_2_index, dn) unless dn.empty?
      end
      index+=1
    end
=end
  end

  def test_print_user_entitlement
    column_num=Array.new
    @equinix_tracker.equinix_user_entitlement.values.map do |rec|
      column_num.push(rec.each.size)
    end
    assert_equal 1, column_num.uniq!.size
    $stdout = StringIO.new
    @equinix_tracker.print_user.each
    items=$stdout.string.split("\n")
    items.each do |line|
      assert_equal column_num[0], line.count("|")
    end
  end

  def test_save_prime_operator_map
    @equinix_tracker.send(:save!, File.dirname(__FILE__)+"/equinix_testfiles/equinix_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/equinix_testfiles/equinix_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# Equinix to AD user map file created by the .+ class .+ method at: .+/), line
      else
        user_name=line.split(":")[0]
        first_name=line.split(":")[1].split("|")[0]
        dn=line.split(":")[1].split("|")[1].strip
        assert_equal @equinix_tracker.equinix_user_entitlement[line_count]["Username:"].upcase,user_name
        assert_equal @equinix_tracker.equinix_user_entitlement[line_count]["First Name:"].upcase, first_name
        if dn.empty?
          assert_nil @equinix_tracker.equinix_user_entitlement[line_count]["DN"]
        else
          assert_equal @equinix_tracker.equinix_user_entitlement[line_count]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/equinix_testfiles/equinix_user_map_test.txt")
  end

  def test_be_search_by_dn
    index=0
    File.open(@equinix_tracker.file_equinix_user_map, 'r').each_line do |line|
    if index>0
      dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
      assert_equal dn, @equinix_tracker.send(:search_by_dn, dn)["DN"] unless dn.empty?
    end
      index=index+1
    end
  end

end
