require "minitest/autorun"
require "ucert"
require 'fileutils'

class TestFisEgiftsTracker < Minitest::Test

    def setup
      @egifts_tracker = Ucert::FisEgiftsTracker.new(:verbose=>false)
    end

    def test_instance_initialize_success
      refute_nil @egifts_tracker.egifts_user_account_function_report
      refute_nil @egifts_tracker.file_user_map
      refute_nil @egifts_tracker.egifts_user_entitlement
      refute_nil @egifts_tracker.egifts_2_ad_user
    end

    def test_egifts_user_account_function_report
      @test_egift_entitlement=@egifts_tracker.send(:parse_egifts_user_account_function_report, File.dirname(__FILE__)+"/fis_egifts_testfiles/CHINA_MERCHANTS_BANK_-_USER_ACCOUNT_FUNCTION_REPORT_TEST.xlsx")
      for i in 1..@test_egift_entitlement.size do
        @test_egift_entitlement[i].each do |key, value|
          if key =="Functions"
            value.each do |function, desc|
              function_key=Array.new
              1.upto(desc.size) do |num|
                function_key.push("TEST"+i.to_s+"-"+num.to_s+"-"+function)
              end
              assert_equal function_key, desc
            end
          else
            if value.include?(' ')
              assert_equal ("TEST"+i.to_s+"_"+key).downcase, value.sub!(' ','_').downcase
            else
              assert_equal ("TEST"+i.to_s+"_"+key).downcase, value.downcase
            end
          end
        end
      end
    end

  def test_insert_dn
    @egifts_tracker.egifts_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

#typo in method, should be "User_Name", not "User_name" in if val['User_name'].upcase == name.upcase, which caused the error and fail
  def test_user_name_2_record_num
    #The test_file would be updated with Active Directory
    index=0
    File.open(@egifts_tracker.file_user_map, 'r').each_line do |line|
      if index>0
        name=line.split(":")[0]
        assert_equal index, @egifts_tracker.send(:user_name_2_record_num, name)
      end
      index=index+1
    end
  end

  def test_print_role_entitlement
    $stdout = StringIO.new
    @egifts_tracker.print_user
    items=$stdout.string.split("\n")
    items.each do |line|
      assert_equal 18, line.count("|")
    end
  end

  def test_save_prime_operator_map
    @egifts_tracker.send(:save!, File.dirname(__FILE__)+"/fis_egifts_testfiles/egifts_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/fis_egifts_testfiles/egifts_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# local egifts to AD user map file created by the .+ class .+ method at: .+/), line
      else
        user_name=line.split(":")[0]
        first_name=line.split(":")[1].split("|")[0].nil? ? first_name="" : first_name=line.split(":")[1].split("|")[0].strip
        dn=line.split(":")[1].split("|")[1].strip
        assert_equal @egifts_tracker.egifts_user_entitlement[line_count]["User_Name"].upcase,user_name
        assert_equal @egifts_tracker.egifts_user_entitlement[line_count]["First_Name"].upcase, first_name
        if dn.empty?
          assert_nil @egifts_tracker.egifts_user_entitlement[line_count]["DN"]
        else
          assert_equal @egifts_tracker.egifts_user_entitlement[line_count]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/fis_egifts_testfiles/egifts_user_map_test.txt")
  end

  def test_egifts_search_by_dn
    index=0
    #puts @egifts_tracker.egifts_user_entitlement
    File.open(@egifts_tracker.file_user_map, 'r').each_line do |line|
    if index>0
      dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
      assert_equal dn, @egifts_tracker.send(:search_by_dn, dn)["DN"] unless dn.empty?
    end
      index=index+1
    end
  end

end
