require "minitest/autorun"
require "ucert"
require 'fileutils'

#overriding initialize method for loading test_files
class TestPrime < Ucert::FisPrimeTracker

  attr_accessor  :verbose, :prime_operator_status_report, :prime_operator_right_report, :file_user_map
  attr_reader     :prime_2_ad_user, :prime_operator_entitlement, :prime_role_entitlement

  def initialize (params ={})
    @verbose=params.fetch(:verbose, false)
    @prime_operator_status_report = File.dirname(__FILE__)+"/fis_prime_testfiles/Prime_Operator_Status_Report_Test.xml"
    @prime_operator_rights_report = File.dirname(__FILE__)+"/fis_prime_testfiles/Prime_Operator_Rights_Report_Test.xml"
    @prime_rights_by_role_report = File.dirname(__FILE__)+"/fis_prime_testfiles/Prime_Rights_by_Role_Report_Test.xml"
    @prime_operators_roles_report = File.dirname(__FILE__)+"/fis_prime_testfiles/Prime_Operators_and_Roles_Report_Test.xml"
    @file_prime_user_map =  File.dirname(__FILE__)+"/fis_prime_testfiles/prime_user_map.txt"
    @prime_2_ad_user=load_known_user_map_from_file(@file_prime_user_map)
    parse_operator_status_report(@prime_operator_status_report)
    parse_operator_right_report(@prime_operator_rights_report)
    parse_operators_and_roles_report(@prime_operators_roles_report)
    insert_dn
    save!
    @prime_role_entitlement=parse_rights_by_role_report(@prime_rights_by_role_report)
  end

end

class TestFisPrimeTracker < Minitest::Test

    def setup
      @test_prime_tracker = TestPrime.new(:verbose=>false)
      @prime_tracker = Ucert::FisPrimeTracker.new(:verbose=>false)
    end

    def test_instance_initialize_success
      refute_nil @prime_tracker.prime_operator_status_report
      refute_nil @prime_tracker.prime_2_ad_user
      refute_nil @prime_tracker.prime_operator_entitlement
      refute_nil @prime_tracker.prime_role_entitlement
    end

    def test_parse_operator_status_report
      #doc = File.open(File.dirname(__FILE__)+"/fis_prime_testfiles/Prime_Operator_Status_Report_Copy.xml",'r')
      doc = File.open(File.dirname(__FILE__)+"/../data/fis_prime/Prime_Operator_Status_Report.xml",'r')
      index=0
      index_trans=Hash.new
      @test_operator_status=Array.new
      doc.each do |line|
        if line.include?("Details Level")
          index+=1
          @test_operator_status[index]=Hash.new
        end
        if line.include?("FormattedValue")
          attribute=line[/.*1.(.*)\}/,1]
          value=line[/.*\<Value\>(.*)\<\/Value\>/,1]
            case attribute
            when "Code"
              @test_operator_status[index].merge!({"Operator.Code"=>value})
              operator_index=@prime_tracker.send(:code_2_index, value)
              index_trans.merge!({index=>operator_index})
            when "UserName"
              @test_operator_status[index].merge!({"Operator.UserName"=>value})
            when "EMail"
              @test_operator_status[index].merge!({"Operator.Email"=>value}) #unless value.nil?
            when "Name"
              @test_operator_status[index].merge!({"Operator.Name"=>value})
            #when "Branch"
            #  @test_operator_status[index].merge!({"Operator.Approved"=>value})
            #when "Dept"
            #  @test_operator_status[index].merge!({"Operator.Approved"=>value})
            when "Approved"
              @test_operator_status[index].merge!({"Operator.Approved"=>value})
            when "ApproveOper"
              @test_operator_status[index].merge!({"Operator.ApproveOper"=>value})
            when "LastApprove"
              @test_operator_status[index].merge!({"Operator.LastApprove"=>value})
            when "ExpirationDate"
              @test_operator_status[index].merge!({"Operator.ExpirationDate"=>value})
            when "LastPwdChgDate"
              @test_operator_status[index].merge!({"Operator.LastPwdChgDate"=>value})
            when "LastPwdChgOper"
              @test_operator_status[index].merge!({"Operator.LastPwdChgOper"=>value})
            when "LastOper"
              @test_operator_status[index].merge!({"Operator.LastOper"=>value})
            when "LastModify"
              @test_operator_status[index].merge!({"Operator.LastModify"=>value})
            when "LastLogin"
              @test_operator_status[index].merge!({"Operator.LastLogin"=>value})
            when "CreateDate"
              @test_operator_status[index].merge!({"Operator.CreateDate"=>value})
            else
              #do nothing
            end
        end
      end
      for i in 1..@test_operator_status.size-1 do
        #some records in opreator status are not consistent with operator_right which will return 0 in index transform
          if index_trans[i]!=0
            @test_operator_status[i].keys.each do |key|
              assert_equal @test_operator_status[i][key], @prime_tracker.prime_operator_entitlement[index_trans[i]][key]
            end
          end
      end
      refute_nil @prime_tracker.prime_role_entitlement
    end

    def test_parse_operator_right_report
      keys=["Operator.Code", "Operator.Name", "Operator.UserName", "Operator.Branch", "Operator.Dept", "Operator.Enabled", "Operator.Deleted","Entitlements"]
      en_keys=["ScopeText", "OperRights.Right", "OperRights.ObjBranch", "OperRights.ObjDept", "OperRights.ObjOper"]
      operator = Array.new
      operator[0]=keys.zip(["ADMIN", "ADMINISTRATOR", "Administrator", "NY", "SEC", "True", "False"]).to_h
      operator1_en1=["Enterprise", "ACPTEDDRFD", "", "", "", "", ""]
      operator1_en2=["Enterprise", "ADDACT", "", "", "", "", ""]
      operator[0]["Entitlements"]=Hash.new
      operator[0]["Entitlements"].merge!({"ACPTEDDRFD"=>en_keys.zip(operator1_en1).to_h})
      operator[0]["Entitlements"].merge!({"ADDACT"=>en_keys.zip(operator1_en2).to_h})
      operator[1]=keys.zip(["AUDIT","INTERNAL AUDIT","AUDIT","NY","1","False","False"]).to_h
      operator2_en1=["Enterprise", "AUDALTARCH", "", "", "", "", ""]
      operator[1]["Entitlements"]=Hash.new
      operator[1]["Entitlements"].merge!({"AUDALTARCH"=>en_keys.zip(operator2_en1).to_h})
      assert_equal 2, @test_prime_tracker.prime_operator_entitlement.size
      operator.each_with_index do |entitlement,index|
        entitlement.each do |key, value|
          assert_equal value, @test_prime_tracker.prime_operator_entitlement[index+1][key]
        end
      end
    end

    def test_parse_operators_and_roles_report
      #doc = File.open(File.dirname(__FILE__)+"/fis_prime_testfiles/Prime_Operators_and_Roles_Report_Copy.xml",'r')
      doc = File.open(File.dirname(__FILE__)+"/../data/fis_prime/Prime_Operators_and_Roles_Report.xml",'r')
      index=0
      index_trans=Hash.new
      @test_operator_roles=Array.new
      doc.each do |line|
        if line.include?("Group Level")
          index+=1
          @test_operator_roles[index]=Hash.new
        end
        if line.include?("FormattedValue")
          attribute=line[/.*1.(.*)\}/,1]
          value=line[/.*\<Value\>(.*)\<\/Value\>/,1]
            case attribute
            when "Code"
              operator_index=@prime_tracker.send(:code_2_index, value)
              index_trans.merge!({index=>operator_index})
            when "Role"
              @key=value
            when "RoleName"
              @test_operator_roles[index]["Operator.Roles"]=Hash.new unless @test_operator_roles[index]["Operator.Roles"]
              @test_operator_roles[index]["Operator.Roles"].merge!({@key=>value})
            else
            end
        end
      end
      for i in 1..@test_operator_roles.size-1 do
        #some records in opreator status are not consistent with operator_right which will return 0 in index transform
        if index_trans[i]!=0
          @test_operator_roles[i]["Operator.Roles"].keys.each do |key|
            assert_equal @test_operator_roles[i]["Operator.Roles"][key], @prime_tracker.prime_operator_entitlement[index_trans[i]]["Operator.Roles"][key]
          end
        end
      end
      refute_nil @prime_tracker.prime_role_entitlement
    end

#The value of 'ObjBranch' in the entitlement is achieved from the value of 'RightsDesc' in the report, which caused the test fail
    def test_parse_rights_by_role_report
      keys=["RoleCode", "RoleDesc", "Enabled", "Delete", "Approved","Rights"]
      en_keys=["RightsCategory", "RightCode", "RightsDesc", "ObjBranch", "ObjDept", "ObjOper"]
      operator = Array.new
      operator[1]=keys.zip(["CMBCM", "Operator is senior Compliance department staff granted all access with the exception of Security Administration of the system.", "Enabled", "Not Deleted", "Approved"]).to_h
      operator1_en1=["EDD", "ACPTEDDRFD", "Allows bulk acceptance of all risk scores", "", "", ""]
      operator1_en2=["BSA", "ADDACT", "Add activity/Import unprocessed activity", "", "", ""]
      operator[1]["Rights"]=Hash.new
      operator[1]["Rights"].merge!({"ACPTEDDRFD"=>en_keys.zip(operator1_en1).to_h})
      operator[1]["Rights"].merge!({"ADDACT"=>en_keys.zip(operator1_en2).to_h})
      operator[0]=keys.zip(["CMBAUDIT","Auditor has rights to audit Cases and Alerts, view only access to all objects in BSA/EDD, add correspondence, documents, attachments, tasks and export list view data.",
        "Enabled","Not Deleted","Approved"]).to_h
      operator2_en1=["BSA", "AUDALTARCH", "Audit Alert History", "", "", ""]
      operator[0]["Rights"]=Hash.new
      operator[0]["Rights"].merge!({"AUDALTARCH"=>en_keys.zip(operator2_en1).to_h})

      operator.each_with_index do |entitlement,index|
        entitlement.each do |key, value|
          assert_equal value, @test_prime_tracker.prime_role_entitlement[index+1][key]
        end
      end

      @test_prime_tracker.prime_operator_entitlement.each do |index, content|
        content["Operator.Roles"].keys.each do |key|
          @test_prime_tracker.prime_role_entitlement.each do |num, value|
            if value["RoleCode"]==key
              value["Rights"].each do |name, desc|
                assert_equal content["Entitlements"][name]["OperRights.Right"], name
              end
            end
          end
        end
      end
    end

    def test_code_2_index
      FileUtils.cp_r(File.dirname(__FILE__)+"/../data/fis_prime/prime_user_map.txt", File.dirname(__FILE__)+"/fis_prime_testfiles/prime_user_map_test.txt")
      index=0
      File.open(File.dirname(__FILE__)+"/fis_prime_testfiles/prime_user_map_test.txt", 'r').each_line do |line|
        if index>0
          code=line.split(":")[0]
          assert_equal index, @prime_tracker.send(:code_2_index, code) unless @prime_tracker.send(:code_2_index, code).zero?
        end
        index=index+1
      end
      File.delete(File.dirname(__FILE__)+"/fis_prime_testfiles/prime_user_map_test.txt")
    end

    def test_insert_dn
      @prime_tracker.prime_operator_entitlement.each do |index, user|
        dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
        assert_match dn_regex, user["DN"] unless user["DN"].nil?
      end
    end

    def test_print_role_entitlement
      $stdout = StringIO.new
      @prime_tracker.print_role
      items=$stdout.string.split("\n")
      line_cnt=1
      items.each do |line|
        if line_cnt>1
          assert_equal 11, line.count("|")
        else
          assert_equal 5, line.count("|")
        end
      end
    end

    def test_print_operator_entitlement
      $stdout = StringIO.new
      @prime_tracker.print_oper
      items=$stdout.string.split("\n")
      line_cnt=1
      items.each do |line|
        if line_cnt>1
          assert_equal 11, line.count("|")
        else
          assert_equal 11, line.count("|")
        end
      end
    end

    def test_save_prime_operator_map
      @prime_tracker.send(:save!, File.dirname(__FILE__)+"/fis_prime_testfiles/prime_user_map_test.txt")
      line_count=0
      File.open(File.dirname(__FILE__)+"/fis_prime_testfiles/prime_user_map_test.txt", 'r').each_line do |line|
        if line_count==0
          assert_match (/\# local Prime to AD user map file created by the .+ class .+ method at: .+/), line
        else
          code=line.split(":")[0]
          line.split(":")[1].split("|")[0].nil? ? user_name="" : user_name=line.split(":")[1].split("|")[0].strip
          line.split(":")[1].split("|")[1].nil? ? dn="" : dn=line.split(":")[1].split("|")[1].strip
          assert_equal @prime_tracker.prime_operator_entitlement[line_count]["Operator.Code"].upcase,code
          assert_equal @prime_tracker.prime_operator_entitlement[line_count]["Operator.UserName"].upcase, user_name
          if dn.empty?
            assert_nil @prime_tracker.prime_operator_entitlement[line_count]["DN"]
          else
            assert_equal @prime_tracker.prime_operator_entitlement[line_count]["DN"], dn
          end
        end
        line_count+=1
      end
      File.delete(File.dirname(__FILE__)+"/fis_prime_testfiles/prime_user_map_test.txt")
    end

  def test_fis_prime_search_by_dn
    FileUtils.cp_r(File.dirname(__FILE__)+"/../data/fis_prime/prime_user_map.txt", File.dirname(__FILE__)+"/fis_prime_testfiles/prime_user_map_test.txt")
    index=0
    File.open(File.dirname(__FILE__)+"/fis_prime_testfiles/prime_user_map_test.txt", 'r').each_line do |line|
    if index>0
      dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
      assert_equal dn, @prime_tracker.send(:search_by_dn, dn)["DN"] unless dn.empty?#||@prime_tracker.send(:search_by_dn, dn).nil?
    end
      index=index+1
    end
    File.delete(File.dirname(__FILE__)+"/fis_prime_testfiles/prime_user_map_test.txt")
  end

end
