require "minitest/autorun"
require "ucert"

class TestFrbTracker < Minitest::Test

  def setup
    @frb_tracker = Ucert::FrbTracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
     refute_nil @frb_tracker.frb_subscriber_roles_report
     refute_nil @frb_tracker.file_user_map
     refute_nil @frb_tracker.frb_user_entitlement
     refute_nil @frb_tracker.frb_2_ad_user
  end

  def  test_parse_frb_rpt_method
    date_time =/^(0?[1-9]|1[12]$)\/(0?[1-9]|[12]\d|3[01])\/([1-9]\d\d\d) ([01]\d|2[0-3])\:([0-5]\d)\:([0-5]\d)$/
    @frb_test_entitlement=@frb_tracker.send(:parse_frb_rpt, File.dirname(__FILE__)+"/frb_testfiles/FRB_Subscriber_Roles_Report_Test.xlsx")
    assert_equal 3, @frb_test_entitlement.size
    @frb_test_entitlement.each do |index, people|
      assert_equal true, people.has_key?("User Name")
      people.each do |key, value|
        if key=="Credential Type:"
          assert ["certificate", "token"].include? value
        elsif key=="Credential Status:"
          assert ["Active","N/A"].include? value
        elsif key=="Credential Created:" || key == "Credential Expires:"
          assert_match date_time, value
        elsif key=="Email"
          assert_match (/\A[\w+\-.]+@[a-z\d\-]+(\.[a-z]+)*\.[a-z]+\z/i) , value
        elsif key=="Services"
          assert_kind_of Hash ,value
        else
          refute_nil value
        end
      end
    end
  end

#user input has typo about his last name
#YINAN Y LIU:YINANLU@NY.CMBCHINA.COM|CN=Yinan Lu,OU=Users,OU=Treasury,OU=New York Branch,DC=ny,DC=cmbchina,DC=com
  def test_insert_dn
    @frb_tracker.frb_user_entitlement.each do |index, user|
      dn_regex=/\ACN=[A-Z][a-z]+ [A-Z][a-z]+,OU=[a-zA-Z]+,OU=[a-zA-Z]+,OU=New York Branch,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"]
    end
  end

  def test_user_name_2_index
    index=0
    File.open(@frb_tracker.file_user_map, 'r').each_line do |line|
      if index>0
        name=line.split(":")[0]
        assert_equal index, @frb_tracker.send(:user_name_2_index, name)
      end
      index=index+1
    end
    assert_operator 1, :<, index
  end

  def test_print_user_entitlement
    $stdout = StringIO.new
    @frb_tracker.print_user
    items=$stdout.string.split("\n")
    items.each do |line|
      assert_match (/\A.+\|.+\|.+\|[\w+\-.]+@[a-z\d\-]+(\.[a-z]+)*\.[a-z]+\|\{.+\}\|.*\|\z/), line
    end
  end

  def test_save_frb_user_map!
    @frb_tracker.send(:save!, File.dirname(__FILE__)+"/frb_testfiles/frb_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/frb_testfiles/frb_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# local Frb to AD user map file created by the .+ class .+ method at: .+/), line
      else
        user_name=line.split(":")[0]
        email=line.split(":")[1].split("|")[0]
        #dn=line.split(":")[1].split("|")[1].chomp
        dn=line.split(":")[1].split("|")[1].nil? ? dn="" : dn=line.split(":")[1].split("|")[1].strip
        assert_equal @frb_tracker.frb_user_entitlement[line_count]["User Name"].upcase, user_name
        assert_equal @frb_tracker.frb_user_entitlement[line_count]["Email"].upcase, email
        assert_equal @frb_tracker.frb_user_entitlement[line_count]["DN"], dn
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/frb_testfiles/frb_user_map_test.txt")
  end

  def test_frb_search_by_dn
    index=0
    File.open(@frb_tracker.file_user_map, 'r').each_line do |line|
    #File.open("./frb_testfiles/frb_user_map_test.txt", 'r').each_line do |line|
    # search according to private variable @frb_user_entitlement, so cannot use test_map.
      if index>0
          line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @frb_tracker.send(:search_by_dn, dn)['DN']
      end
      index=index+1
    end
  end

end
