require "minitest/autorun"
require "ucert"

class TestJpmAccessTracker < Minitest::Test
  def setup
    @jpm_tracker = Ucert::JpmAccessTracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
    refute_nil @jpm_tracker.file_jpm_user_map
    refute_nil @jpm_tracker.jpm_2_ad_user
    refute_nil @jpm_tracker.jpm_user_entitlement
  end

  # infromation in the first row is not recorded except first user, which causes failure.
  def test_parse_jpm_user_entitlement_report
=begin #prepare test_file, only uncomment it when test_file is empty
    doc = File.open(File.dirname(__FILE__)+"/jpm_access_testfiles/jpm_user_entitlements_details_test.txt",'w')
    doc.write("User First Name|User Middle Initial|User Last Name|Job Title|User ID|Logon Type|Employee ID|User Status|Email|Address Line 1|Address Line 2|City|State/Province/Region|Zip/Postal Code|Country|Phone 1|Phone 1 Extension|Phone 1 Type|Phone 2|Phone 2 Extension|Phone 2 Type|Phone 3|Phone 3 Extension|Phone 3 Type|Last Logon Date|Last Modified Date|Last Modified By|Member of User Group Name|Member of User Group Description|Product Name|Product Entitlement Method|Sub Product Name|Sub Product Entitlement Method|Function Type|Function Name|Function Name Entitlement Method|Function Status|Limit Product|User Attribute|Payment/Transaction Limit|Payment/Transaction Limit Entitlement Method|Batch Limit|Batch Limit Entitlement Method|Daily Limit|Daily Limit Entitlement Method|Acct Group Name|Acct Group Entitlement Method|FX Acct Group Name|FX Acct Group Entitlement Method|Eligible For|Acct Name/NickName|Acct Number/TPA/Program ID|Account Entitlement Method|Parent Billing Group Number|Parent Billing Group Entitlement Method|Billing Account Group Name|Billing Account Name|Billing Account Entitlement Method|Confidential Transaction|Bank|Currency|Account Transfer - Transfer Direction|All ACH Credit Limit|Credit Limit Entitlement Method|All ACH Debit Limit|Debit Limit Entitlement Method|FX Daily Trans. Limit|FX Daily Trans. Limit Currency|FX Daily Limit Entitlement Method|FX Individual Trans. Limit|FX Individual Trans. Limit Currency|FX Individual Limit Entitlement Method|ACH Company Name|ACH Company ID|ACH Company Entitlement Method|Arrangement Name|Currency Name|Restricted Country Entitlement Method|More Services|Description Country|View Payroll Summary|")
    doc.write("\n")
    for kk in 1..3 do
      for ii in 1..kk do
        for jj in 1..27 do
          doc.write("Test"+kk.to_s+":"+jj.to_s+"|")
        end
        for jj in 28..79 do
          doc.write("Test"+kk.to_s+":"+ii.to_s+":"+jj.to_s+"|")
        end
        doc.write("\n")
      end
    end
   doc.close
=end
    @test_entitlement= @jpm_tracker.send(:parse_jpm_user_entitlement_report, File.dirname(__FILE__)+"/jpm_access_testfiles/jpm_user_entitlements_details_test.txt")
    assert_equal 3, @test_entitlement.size
    #puts @test_entitlement
    #count=1
    for kk in 1..3 do
      for ii in 1..kk do
        for jj in 1..27 do
        #  puts count
          assert_equal "Test"+kk.to_s+":"+jj.to_s, @test_entitlement[kk].values[jj-1]
        #  count+=1
        end
        for jj in 28..79 do
        #  puts count
          assert_equal "Test"+kk.to_s+":"+ii.to_s+":"+jj.to_s, (@test_entitlement[kk].values[27][ii-1].values)[jj-28]
        #  count+=1
        end
      end
    end
  end

  def test_dn_2_index
=begin
    FileUtils.cp_r(File.dirname(__FILE__)+"/../data/jpm_access/jpm_access_user_map.txt", File.dirname(__FILE__)+"/jpm_access_testfiles/jpm_access_user_map_test.txt")
    line_cnt=1
    File.open(File.dirname(__FILE__)+"/jpm_access_testfiles/jpm_access_user_map_test.txt", 'r').each_line do |line|
      if line_cnt>1
        dn=line.split("|")[1].strip
        assert_equal line_cnt-1, @jpm_tracker.send(:dn_2_index, dn)
      end
      line_cnt+=1
    end
    File.delete(File.dirname(__FILE__)+"/jpm_access_testfiles/jpm_access_user_map_test.txt")
=end
  end

  def test_insert_dn
    @jpm_tracker.jpm_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

  def test_print_user_entitlement
      column_num=Array.new
      @jpm_tracker.jpm_user_entitlement.values.map do |rec|
        column_num.push(rec.each.size)
      end
      assert_equal 1, column_num.uniq!.size
      $stdout = StringIO.new
      @jpm_tracker.print_user.each
      items=$stdout.string.split("\n")
      items.each do |line|
       assert_equal column_num[0]-1, line.count("|")
      end
  end

  def test_save_jpm_user_map!
    @jpm_tracker.send(:save!, File.dirname(__FILE__)+"/jpm_access_testfiles/jpm_access_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/jpm_access_testfiles/jpm_access_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# local Jpm for Securitites to AD user map file created by the .+ class .+ method at: .+/), line
      else
        user_id=line.split("|")[0]
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal @jpm_tracker.jpm_user_entitlement[line_count]["User ID"].upcase, user_id
        if dn.empty?
          assert_nil @jpm_tracker.jpm_user_entitlement[line_count]["DN"]
        else
          assert_equal @jpm_tracker.jpm_user_entitlement[line_count]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/jpm_access_testfiles/jpm_access_user_map_test.txt")
  end

  def test_jpm_search_by_dn
    index=0
    File.open(@jpm_tracker.file_jpm_user_map, 'r').each_line do |line|
      if index>0
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @jpm_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
      end
      index=index+1
    end
  end

end
