require "minitest/autorun"
require "ucert"

class TestMadison535Tracker < Minitest::Test

  def setup
    @madison535_tracker = Ucert::Madison535Tracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
     refute_nil @madison535_tracker.madison535_user_access_report
     refute_nil @madison535_tracker.madison535_user_access_report_2
     refute_nil @madison535_tracker.file_madison535_user_map
     refute_nil @madison535_tracker.madison535_2_ad_user
     refute_nil @madison535_tracker.madison535_user_access
  end

  def test_parse_madison535_user_access_report_method
    @madison535_user_access_repot_test=@madison535_tracker.send(:parse_madison535_user_access_report, File.dirname(__FILE__)+"/madison535_testfiles/535madison_bldg_pass_Test.xlsx")
    users=Array.new
    for ii in 1..3 do
      users.push(["CHINA MERCHANTS BANK","#{ii}","test#{ii}_firstname","test#{ii}_lastname"])
    end
    ii=0
    @madison535_user_access_repot_test.each do |index, content|
      assert_equal users[ii], content.values.to_a
      ii+=1
    end
  end

  def test_parse_madison535_user_access_report_2_method
    @madison535_tracker.send(:parse_madison535_user_access_report_2, File.dirname(__FILE__)+"/madison535_testfiles/535madison_bldg_pass_2_Test.xlsx")
    user1={"First Name"=>"test1_firstname","Last Name"=>"test1_lastname",	"Card #"=>"1",	"Clearance1"=>"",	"Clearance2"=>"Main Door - 24x7x365",	"Clearance3"=>"",	"Clearance4"=>"Trading Room - 24x7x365",	"Clearance5"=>"",	"Disabled"=>"no","Company"=>"CMBNY", "DN"=>nil}
    user2={"First Name"=>"test2_firstname","Last Name"=>"test2_lastname",	"Card #"=>"1",	"Clearance1"=>"IT Room Door - 24x7x365",	"Clearance2"=>"",	"Clearance3"=>"Security Room - 24x7x365",	"Clearance4"=>"",	"Clearance5"=>"FL-23 File Room Access",	"Disabled"=>"yes","Company"=>"CMBNY", "DN"=>nil}
    user3={"First Name"=>"test3_firstname","Last Name"=>"test3_lastname",	"Card #"=>"2",	"Clearance1"=>"",	"Clearance2"=>"",	"Clearance3"=>"",	"Clearance4"=>"",	"Clearance5"=>"",	"Disabled"=>"yes","Company"=>"CMBNY", "DN"=>nil}
    last_index=@madison535_tracker.madison535_user_access.size
    assert_equal user3, @madison535_tracker.madison535_user_access[last_index]
    assert_equal user2, @madison535_tracker.madison535_user_access[last_index-1]
    #user1 would be overwritten by user2 since they have same card number.
    refute_equal user1, @madison535_tracker.madison535_user_access[last_index-2]
  end

#function is not in use, but mutiple indexes map to one DN, which would cause test fail
  def test_dn_2_index
=begin
    FileUtils.cp_r(File.dirname(__FILE__)+"/../data/madison535/madison535_user_map.txt", File.dirname(__FILE__)+"/madison535_testfiles/madison535_user_map_test.txt")
    line_cnt=1
    File.open(File.dirname(__FILE__)+"/madison535_testfiles/madison535_user_map_test.txt", 'r').each_line do |line|
      if line_cnt>1
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal line_cnt-1, @madison535_tracker.send(:dn_2_index, dn) unless dn.empty?
      end
      line_cnt+=1
    end
    File.delete(File.dirname(__FILE__)+"/madison535_testfiles/madison535_user_map_test.txt")
=end
  end

  def test_card_num_2_index
    FileUtils.cp_r(File.dirname(__FILE__)+"/../data/madison535/madison535_user_map.txt", File.dirname(__FILE__)+"/madison535_testfiles/madison535_user_map_test.txt")
    line_cnt=1
    File.open(File.dirname(__FILE__)+"/madison535_testfiles/madison535_user_map_test.txt", 'r').each_line do |line|
      if line_cnt>1
        card_num=line.split("|")[0].strip
        assert_equal line_cnt-1, @madison535_tracker.send(:card_num_2_index, card_num)
      end
      line_cnt+=1
    end
    File.delete(File.dirname(__FILE__)+"/madison535_testfiles/madison535_user_map_test.txt")
  end

  def test_insert_dn
    @madison535_tracker.madison535_user_access.each do |index, user|
      dn_regex=/\ACN=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

  def test_print_user_access
    column_num=Array.new
    @madison535_tracker.madison535_user_access.values.map do |rec|
      column_num.push(rec.each.size)
    end
    column_num.uniq!
    $stdout = StringIO.new
    @madison535_tracker.print_user.each
    items=$stdout.string.split("\n")
    items.each do |line|
     assert_includes column_num, line.count("|")
    end
  end

  def test_save_madison535_user_map!
    @madison535_tracker.send(:save!, File.dirname(__FILE__)+"/madison535_testfiles/madison535_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/madison535_testfiles/madison535_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# Madison535 to AD user map file created by the .+ class .+ method at: .+/), line
      else
        card_num=line.split("|")[0]
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal @madison535_tracker.madison535_user_access[line_count]["Card #"].upcase, card_num
        if dn.empty?
          assert_nil @madison535_tracker.madison535_user_access[line_count]["DN"]
        else
          assert_equal @madison535_tracker.madison535_user_access[line_count]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/madison535_testfiles/madison535_user_map_test.txt")
  end

  def test_madison535_search_by_dn
    index=0
    File.open(@madison535_tracker.file_madison535_user_map, 'r').each_line do |line|
      if index>0
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @madison535_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
      end
      index=index+1
    end
  end

end
