require "minitest/autorun"
require "ucert"

class TestMantisTracker < Minitest::Test

  def setup
    @mantis_tracker = Ucert::MantisTracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
     refute_nil @mantis_tracker.file_mantis_user_map
     refute_nil @mantis_tracker.mantis_2_ad_user
     refute_nil @mantis_tracker.mantis_user_entitlement
     refute_nil @mantis_tracker.mantis_access_level
  end

  def test_parse_mantis_user_report_method
    @mantis_test_entitlement=@mantis_tracker.send(:parse_mantis_user_report, File.dirname(__FILE__)+"/mantis_testfiles/Mantis_ActiveUsers_Rpt_Test.xlsx")
    #Load access levels
    access_level=Array.new
    workbook = RubyXL::Parser.parse(File.dirname(__FILE__)+"/mantis_testfiles/Mantis_AccessLevels_Test.xlsx")
    worksheet = workbook[0]
    worksheet.count.times do |row|
      if worksheet[row][0].value.to_s.match(/\A(\d)+\z/)
       access_level.push(worksheet[row][0].value.to_s)
      end
    end
    access_level.uniq!
    assert_equal 4, @mantis_test_entitlement.size
    @mantis_test_entitlement.each do |index, people|
      people.each do |key, value|
        if key=="id"
          assert_match (/\A(\d)+\z/), value
        elsif key=="username"
          refute_nil value
        elsif key=="access_level"
          assert access_level.include? value
        elsif key=="email"
          assert_match (/\A[\w+\-.]+@[a-z\d\-]+(\.[a-z]+)*\.[a-z]+\z/i), value unless value=="root@localhost"
        end
      end
    end
  end

  def test_parse_mantis_access_level_report
    @mantis_test_level=@mantis_tracker.send(:parse_mantis_access_level_report, File.dirname(__FILE__)+"/mantis_testfiles/Mantis_AccessLevels_Test.xlsx")
    level_group=Array.new
    level_group[0]=["10","Viewer","Can only browse and view bug listings (e.g. guest account)"]
    level_group[1]=["20","Reporter","Can report new bugs and add bug notes"]
    level_group[2]=["30","Updater","Same as reporter and can update bugs (e.g. trusted testers, power users)"]
    level_group[3]=["40","Developer","More privileges than updater. People actually working on a project"]
    level_group[4]=["50","Administrator","All powerful account. Can create accounts, reset passwords, add/remove users, and more"]
    ii=0
    @mantis_test_level.each do |index, content|
      jj=0
      content.each do |key, value|
        assert_equal level_group[ii][jj].strip, value.strip
        jj+=1
      end
      ii+=1
    end
  end

  def test_dn_2_index
    index=1
    File.open(@mantis_tracker.file_mantis_user_map, 'r').each_line do |line|
      if index>1
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal index-1, @mantis_tracker.send(:dn_2_index, dn) unless dn.empty?
      end
      index+=1
    end
  end

  def test_insert_dn
    @mantis_tracker.mantis_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

  def test_print_user_entitlement
    $stdout = StringIO.new
    @mantis_tracker.print_user
    items=$stdout.string.split("\n")
    items.each do |line|
      assert_match (/\A(\d)+\|[a-zA-Z\s]+\|.+\|(\d)+\|[\w+\-.]+@[a-z\d\-]+(\.[a-z]+)*\.[a-z]+\|.*\|\z/), line unless line.include?("root@localhost")
    end
  end

  def test_pirnt_access_level
    $stdout = StringIO.new
    @mantis_tracker.print_acl
    items=$stdout.string.split("\n")
    items.each do |line|
      assert_match (/\A(\d)+\|[a-zA-Z]+\|.+\z/), line
    end
  end

  def test_save_mantis_user_map!
    @mantis_tracker.send(:save!, File.dirname(__FILE__)+"/mantis_testfiles/mantis_access_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/mantis_testfiles/mantis_access_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# local CMBNY Mantis to AD user map file created by the .+ class .+ method at: .+/), line
      else
        user_name=line.split(":")[0]
        email=line.split(":")[1].split("|")[0]
        dn=line.split(":")[1].split("|")[1].nil? ? dn="" : dn=line.split(":")[1].split("|")[1].strip
        assert_equal @mantis_tracker.mantis_user_entitlement[line_count]["username"].upcase, user_name
        assert_equal @mantis_tracker.mantis_user_entitlement[line_count]["email"].upcase, email
        if dn.empty?
          assert_nil @mantis_tracker.mantis_user_entitlement[line_count]["DN"]
        else
          assert_equal @mantis_tracker.mantis_user_entitlement[line_count]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/mantis_testfiles/mantis_access_user_map_test.txt")
  end

  def test_wms_search_by_dn
    index=0
    File.open(@mantis_tracker.file_mantis_user_map, 'r').each_line do |line|
      if index>0
        line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @mantis_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
      end
      index=index+1
    end
  end
end
