require "minitest/autorun"
require "ucert"

class TestSage100Tracker < Minitest::Test

  def setup
   @sage100_tracker = Ucert::Sage100Tracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
     refute_nil @sage100_tracker.sy_user_details_report
     refute_nil @sage100_tracker.file_user_map
     refute_nil @sage100_tracker.sy_user_entitlement
     refute_nil @sage100_tracker.sy_2_ad_user
  end

  def test_parse_sy_user_role_preferences_detail_report
     @test_user_entitlement_1=@sage100_tracker.send(:parse_sy_user_role_preferences_detail_report,File.dirname(__FILE__)+"/sage100_testfiles/SY_UserReport_RolePreferencesDetails_Test.xlsx")
     user_group=Array.new
     user_group[1]={"User Logon"=>"Test", "First Name"=>"Test", "Last Name"=>"", "Active"=>"Active", "User Code"=>"Test", "Customization Group"=>"", "Expires"=>"1899-12-31T00:00:00+00:00", "User Account Locked"=>"No", "E-mail"=>"test@abc.com",
       "Job Title"=>"Test", "Role"=>{"Test User"=>"Test User"}, "Scope"=>{"Organization"=>"all companies", "Start"=>"1899-12-31T00:00:00+00:00", "Expires"=>"1899-12-31T00:00:00+00:00"},
       "Preferences"=>{"Automatic Logoff"=>"No", "Delay in Minutes"=>"", "Implied Decimal Point"=>"Yes", "Use Graphic Report Format"=>"Yes", "Low Speed Connection"=>"No", "Partial Lookup Default"=>"Begins with", "Lookup Limit for Initial Display"=>"0", "Prompt for Company Code"=>"No", "Enter Key Like Tab Key for Grid"=>"Yes", "Default Report Preview Zoom"=>"100%",
       "Display Reduction Amounts In Red"=>"Yes", "Enable C/S ODBC Driver "=>"No", "Spawn Tasks from App Server  "=>"Yes"}}
     user_group[2]={"User Logon"=>"Foob", "First Name"=>"Foo", "Last Name"=>"Bar", "Active"=>"Active", "User Code"=>"FB", "Customization Group"=>"CMB", "Expires"=>"1899-12-31T00:00:00+00:00", "User Account Locked"=>"No", "E-mail"=>nil,
       "Job Title"=>"IT Analyst", "Role"=>{"ISO"=>"Risk Mgmt - Information Security Officer"}, "Scope"=>{"Organization"=>"(CMB) China Merchant Bank", "Start"=>"1899-12-31T00:00:00+00:00", "Expires"=>"1899-12-31T00:00:00+00:00"},
       "Preferences"=>{"Automatic Logoff"=>"No", "Delay in Minutes"=>"", "Implied Decimal Point"=>"Yes", "Use Graphic Report Format"=>"Yes", "Low Speed Connection"=>"No", "Partial Lookup Default"=>"Begins with", "Lookup Limit for Initial Display"=>"0", "Prompt for Company Code"=>"No", "Enter Key Like Tab Key for Grid"=>"Yes", "Default Report Preview Zoom"=>"100%",
       "Display Reduction Amounts In Red"=>"Yes", "Enable C/S ODBC Driver "=>"No", "Spawn Tasks from App Server  "=>"No"}}
     for i in 1..2
       diff=user_group[i].to_a-@test_user_entitlement_1[i].to_a
       puts diff
       assert_equal user_group[i], @test_user_entitlement_1[i]
     end
  end

  #please notice Sarah Han's profile and rows are merged which caused the test fail
  def test_parse_sy_uer_role_task_permissions_detail_report
=begin
    #test_aim: each user should have certain numbers operation of each company
    privilege_type=["Update","Print","Create","Modify","Remove","View","Access"]
    @sage100_tracker.sy_user_entitlement.each do |index, user|
      row=0
      flag=false
      user["Permissions"].each do |operation, privilege|
        if operation=="Accounts Payable"
          flag=true
        end
        if flag==true
          #failure when privilege_type is not included in privilege
          assert privilege_type.any? {|p| privilege.include?(p)} unless privilege.nil?
          row+=1
        end
      end
      puts "Checking user permissions on: #{user["User Logon"]}"
      assert_equal 505, row
    end
=end
  end

  def test_insert_dn
    @sage100_tracker.sy_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

  def test_user_name_2_index
    FileUtils.cp_r(File.dirname(__FILE__)+"/../data/sage100/sy_user_map.txt", File.dirname(__FILE__)+"/sage100_testfiles/sy_user_map_test.txt")
    line_cnt=1
    File.open(File.dirname(__FILE__)+"/sage100_testfiles/sy_user_map_test.txt", 'r').each_line do |line|
      if line_cnt>1
        username=line.split("|")[0].strip
        assert_equal line_cnt-1, @sage100_tracker.send(:user_name_2_index, username)
      end
      line_cnt+=1
    end
    File.delete(File.dirname(__FILE__)+"/sage100_testfiles/sy_user_map_test.txt")
  end

  def test_print_user_entitlement
    $stdout = StringIO.new
    @sage100_tracker.print_user
    items=$stdout.string.split("\n")
    items.each do |line|
      assert_match (/\A.+\|.+\|.+\|\|.+\|.+\|.+\|.+\|\|{.+}\|.*|\z/), line
    end
  end

  def test_save_swo_user_map!
    @sage100_tracker.send(:save!, File.dirname(__FILE__)+"/sage100_testfiles/sy_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/sage100_testfiles/sy_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# local Sage100 to AD user map file created by the .+ class .+ method at: .+/), line
      else
        username=line.split("|")[0]
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal @sage100_tracker.sy_user_entitlement[line_count]["User Logon"].upcase, username
        if dn.empty?
          assert_nil @sage100_tracker.sy_user_entitlement[line_count]["DN"]
        else
          assert_equal @sage100_tracker.sy_user_entitlement[line_count]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/sage100_testfiles/sy_user_map_test.txt")
  end

  def test_sage100_search_by_dn
    index=0
    File.open(@sage100_tracker.file_user_map, 'r').each_line do |line|
      if index>0
        dn=line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @sage100_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
      end
      index=index+1
    end
  end

end
