require "minitest/autorun"
require "ucert"

#overriding initialize method for loading test_files
class TestSom < Ucert::SomTracker

  attr_accessor  :verbose, :som_user_report_test, :file_user_map_test
	attr_reader    :som_user_entitlement, :som_2_ad_user_test

  def initialize(params = {})
    @verbose = params.fetch(:verbose, true)
    # Som to AD user map file
    @file_user_map_test = File.dirname(__FILE__)+"/som_testfiles/som_user_map_test.txt"
    @som_user_report_test = File.dirname(__FILE__)+"/som_testfiles/som_user_report_test.csv"
    # Load the user map file to an instance variable (for performance gain)
    @som_2_ad_user_test = load_known_user_map_from_file(@file_user_map_test)
    # Load the user entitlement instance variable from the native Som user entitlement report
    parse_som_user_report(@som_user_report_test)
    # Insert DN field into the user entitlement data structure
    insert_dn
    save!
  end

  def insert_dn
		puts "Insert DN into the record ..."
		tracker = Ucert::AdTracker.new(:verbose=>@verbose)
		@som_user_entitlement.delete(nil)
		@som_user_entitlement.each do |index, record|
		next if record.nil?
			puts "\n\nPerform DN lookup for record: #{record['uid']}"
			key1 = record['uid'] unless record['uid'].nil?
			next if key1.nil?
			if key1 =~ /\\/
				key2 = key1.split('\\')[1]
			end
			my_key = key1.upcase
			puts "Perform 1st order search from the local cache: #{my_key}"
			dn=@som_2_ad_user_test[my_key]
      @som_user_entitlement[index]['DN'] = dn
	    tracker=nil
    end
	end
end

class TestSomTracker < Minitest::Test

  def setup
    @test_som_tracker = TestSom.new(:verbose=>false)
  end

  def test_parse_som_user_report
    assert_equal 1, @test_som_tracker.som_user_entitlement[1]['systems'].size
    assert_equal 'admin', @test_som_tracker.som_user_entitlement[1]['uid']
    assert_equal 10, @test_som_tracker.som_user_entitlement[1]['systems'][0].size
    assert_equal 4, @test_som_tracker.som_user_entitlement[2]['systems'].size
    assert_equal 'james', @test_som_tracker.som_user_entitlement[2]['uid']
    assert_equal 10, @test_som_tracker.som_user_entitlement[2]['systems'][2].size
    assert_equal 1, @test_som_tracker.som_user_entitlement[3]['systems'].size
    assert_equal 'aluser1', @test_som_tracker.som_user_entitlement[3]['uid']
    assert_equal 5, @test_som_tracker.som_user_entitlement[4]['systems'].size
    assert_equal 7, @test_som_tracker.som_user_entitlement[5]['systems'].size
    assert_equal 6, @test_som_tracker.som_user_entitlement[6]['systems'].size
    assert_equal 10, @test_som_tracker.som_user_entitlement[6]['systems'][4].size
  end

  def test_insert_dn
    @test_som_tracker.som_user_entitlement.each do | index, user |
      assert_equal user['uid'], user['DN'].downcase
    end
  end
end
