require "minitest/autorun"
require "ucert"

class TestT24Tracker < Minitest::Test

  def setup
    @t24_tracker = Ucert::T24Tracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
     refute_nil @t24_tracker.t24_user_entitlement_report
     refute_nil @t24_tracker.t24_group_entitlement_report
     refute_nil @t24_tracker.file_user_map
     refute_nil @t24_tracker.t24_user_entitlement
     refute_nil @t24_tracker.t24_group_entitlement
     refute_nil @t24_tracker.t24_2_ad_user
  end

  def test_parse_t24_user_report
    @test_t24_entitlement=@t24_tracker.send(:parse_t24_user_report, File.dirname(__FILE__)+"/t24_testfiles/T24_User_Rpt_Test.csv")
    titles=File.open(File.dirname(__FILE__)+"/t24_testfiles/T24_User_Rpt_Test.csv", 'r').first.split(",").map {|x| x.gsub("\"","").gsub(/\n/, "")}
    user_group=Hash.new
    for i in 1..3
      user_group[i]=Hash.new
      j=1
      titles.each do |title|
        title.tr!('@','')
          if title=="APPLICATION"
            q=j+1
            user_group[i][title]={"TEST#{i}-#{j}"=>"TEST#{i}-#{q}"}
            j+=1
          elsif title=="FUNCTION"
            j+=1
          elsif title=="TIME.LAST.SIGN.ON"
          else
            user_group[i][title]="TEST#{i}-#{j}"
            j+=1
          end
      end
      k=1
      flag=false
      while k<i do
        key=user_group[i]["APPLICATION"].keys[0]+"-"+k.to_s
        user_group[i]["APPLICATION"][key]=key
        k+=1
        flag=true
      end
      if flag
        user_group[i]["GROUP"]=["@TEST#{i}-3"]
        user_group[i]["APPLICATION"].delete(user_group[i]["APPLICATION"].first[0])
      end
      user_group[i].delete(user_group[i].first[0])
      user_group[i]["ID"]="TEST#{i}-1"
    end
    assert_equal user_group, @test_t24_entitlement
  end

  def test_parse_t24_group_report
    @test_t24_entitlement=@t24_tracker.send(:parse_t24_group_report, File.dirname(__FILE__)+"/t24_testfiles/T24_Grp_Rpt_Test.csv")
    titles=["ID","DESCRIPTION","APPLICATION"]
    test_group=Hash.new
    for i in 1..3
      test_group[i]=Hash.new
      j=1
      titles.each do |title|
        if title=="APPLICATION"
          test_group[i]["APPLICATION"]=Hash.new
          application_name="TEST#{i}-#{j}"
          test_group[i]["APPLICATION"][application_name]=Hash.new
          j+=1
          test_group[i]["APPLICATION"][application_name]["VERSION"]="TEST#{i}-#{j}"
          j+=1
          test_group[i]["APPLICATION"][application_name]["FUNCTION"]="TEST#{i}-#{j}"
        else
          test_group[i][title]="TEST#{i}-#{j}"
        end
        j+=1
      end
      k=1
      while k<i do
        application_name=test_group[i]["APPLICATION"].keys[0]+"-"+k.to_s
        test_group[i]["APPLICATION"][application_name]=Hash.new
        test_group[i]["APPLICATION"][application_name]["VERSION"]=""
        test_group[i]["APPLICATION"][application_name]["FUNCTION"]=application_name
        k+=1
      end
    end
    assert_equal test_group, @test_t24_entitlement
    refute_nil @test_t24_entitlement
  end

  def test_insert_dn
    @t24_tracker.t24_user_entitlement.each do |index, user|
      dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
      assert_match dn_regex, user["DN"] unless user["DN"].nil?
    end
  end

  def test_save_t24_user_map!
    @t24_tracker.send(:save!, File.dirname(__FILE__)+"/t24_testfiles/t24_user_map_test.txt")
    line_count=0
    File.open(File.dirname(__FILE__)+"/t24_testfiles/t24_user_map_test.txt", 'r').each_line do |line|
      if line_count==0
        assert_match (/\# local T24 to AD user map file created by the .+ class .+ method at: .+/), line
      else
        user_id=line.split(":")[0]
        user_name=line.split(":")[1].split("|")[0]
        dn=line.split(":")[1].split("|")[1].nil? ? dn="" : dn=line.split(":")[1].split("|")[1].strip
        assert_equal @t24_tracker.t24_user_entitlement[line_count]["USER.NAME"].upcase, user_name
        assert_equal @t24_tracker.t24_user_entitlement[line_count]["ID"].upcase, user_id
        if dn.empty?
          assert_nil @t24_tracker.t24_user_entitlement[line_count]["DN"]
        else
          assert_equal @t24_tracker.t24_user_entitlement[line_count]["DN"], dn
        end
      end
      line_count+=1
    end
    File.delete(File.dirname(__FILE__)+"/t24_testfiles/t24_user_map_test.txt")
  end

  def test_t24_search_by_dn
    index=0
    File.open(@t24_tracker.file_user_map, 'r').each_line do |line|
      if index>0
        line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
        assert_equal dn, @t24_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
      end
      index=index+1
    end
  end

  def test_print_user_entitlement
    $stdout = StringIO.new
    @t24_tracker.print_user.each
    items=$stdout.string.split("\n")
    items.each do |line|
     assert_equal 6, line.count("|")
    end
  end

  def test_print_group_entitlement
    $stdout = StringIO.new
    @t24_tracker.print_group.each
    items=$stdout.string.split("\n")
    items.each do |line|
     assert_equal 2, line.count("|")
    end
  end

end
