require "minitest/autorun"
require "ucert"

class TestWmsTracker < Minitest::Test

    def setup
      @wms_tracker = Ucert::WmsTracker.new(:verbose=>false)
    end

    def test_instance_initialize_success
       refute_nil @wms_tracker.wms_user_report
       refute_nil @wms_tracker.wms_role_report
       refute_nil @wms_tracker.file_user_map
       refute_nil @wms_tracker.wms_2_ad_user
       refute_nil @wms_tracker.wms_user_entitlement
       refute_nil @wms_tracker.wms_role_entitlement
    end

    def test_parse_wms_user_report_method
      @test_user_entitlement=@wms_tracker.send(:parse_wms_user_report, File.dirname(__FILE__)+"/wms_testfiles/user_rpt_test.txt")
      test_group=Hash.new
      test_group[1]={"USERID"=>"test","USERNAME"=>"测试员","STATUS"=>"1","ORGNAME"=>"全行"}
      test_group[2]={"USERID"=>"admin","USERNAME"=>"管理员","STATUS"=>"1","ORGNAME"=>"纽约分行信息技术部"}
      test_group[3]={"USERID"=>"developer","USERNAME"=>"开发员","STATUS"=>"0","ORGNAME"=>"纽约分行风险管理部"}
      test_group[1]["ROLEID"]={"0000"=>true}
      test_group[2]["ROLEID"]={"0001"=>true,"0002"=>true,"0003"=>true}
      test_group[3]["ROLEID"]={"0004"=>true}
      for i in 1..3
        assert_equal test_group[i], @test_user_entitlement[i]
      end
    end

    def test_parse_wms_role_report_method
      @test_role_entitlement=@wms_tracker.send(:parse_wms_role_report, File.dirname(__FILE__)+"/wms_testfiles/role_rpt_test.txt")
      test_group=Hash.new
      test_group["0000"]={"ROLENAME"=>"超级测试员"}
      test_group["0001"]={"ROLENAME"=>"测试管理员"}
      test_group["0002"]={"ROLENAME"=>"一般测试员"}
      test_group["0000"]["PRV"]=[{"PRVID"=>"0000_SA1","PRVNAME"=>"Super Admin1","URL"=>"/ad1"},{"PRVID"=>"0000_SA2","PRVNAME"=>"Super Admin2","URL"=>"/ad2"}]
      test_group["0001"]["PRV"]=[{"PRVID"=>"0000_SA2","PRVNAME"=>"Super Admin2","URL"=>"/ad2"},{"PRVID"=>"0001_TA1","PRVNAME"=>"Test Admin1","URL"=>"/ta1"}]
      test_group["0002"]["PRV"]=[{"PRVID"=>"0002_TA2","PRVNAME"=>"Test Admin2","URL"=>"/ta2"}]
      for i in ["0000","0001","0002"]
        assert_equal test_group[i], @test_role_entitlement[i]
      end
    end

    def test_insert_dn
      @wms_tracker.wms_user_entitlement.each do |index, user|
        dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
        assert_match dn_regex, user["DN"] unless user["DN"].nil?
      end
    end

    def test_print_user_entitlement
      $stdout = StringIO.new
      @wms_tracker.print_user
      items=$stdout.string.split("\n")
      items.each do |line|
        if line.include?("CN=")
          assert_match (/\A.+\|.+\|(0|1)\|.+\|\{.+\}\|CN=[a-zA-Z\s]+,OU=[a-zA-Z\s]+,.*DC=ny,DC=cmbchina,DC=com\|\z/), line
        else
          assert_match (/\A.+\|.+\|(0|1)\|.+\|\{.+\}\|\|\z/), line
        end
      end
    end

    def test_print_user_role
      $stdout = StringIO.new
      @wms_tracker.print_role
      items=$stdout.string.split("\n")
      items.each do |line|
        assert_match (/\A.+\|.+\|\[\{(\".+\"\=\>.+\,)*\".+\"\=\>.+\}\]\z/), line
      end
    end

    def test_save_wms_user_map!
      @wms_tracker.send(:save!, File.dirname(__FILE__)+"/wms_testfiles/wms_user_map_test.txt")
      line_count=0
      File.open(File.dirname(__FILE__)+"/wms_testfiles/wms_user_map_test.txt", 'r').each_line do |line|
        if line_count==0
          assert_match (/\# local Wms to AD user map file created by the .+ class .+ method at: .+/), line
        else
          user_name=line.split(":")[0]
          user_id=line.split(":")[1].split("|")[0]
          dn=line.split(":")[1].split("|")[1].nil? ? dn="" : dn=line.split(":")[1].split("|")[1].strip
          assert_equal @wms_tracker.wms_user_entitlement[line_count]["USERNAME"].upcase, user_name
          assert_equal @wms_tracker.wms_user_entitlement[line_count]["USERID"].upcase, user_id
          if dn.empty?
            assert_nil @wms_tracker.wms_user_entitlement[line_count]["DN"]
          else
            assert_equal @wms_tracker.wms_user_entitlement[line_count]["DN"], dn
          end
        end
        line_count+=1
      end
      File.delete(File.dirname(__FILE__)+"/wms_testfiles/wms_user_map_test.txt")
    end

    def test_wms_search_by_dn
      index=0
      File.open(@wms_tracker.file_user_map, 'r').each_line do |line|
        if index>0
          line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
          assert_equal dn, @wms_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
        end
        index=index+1
      end
    end
end
