require "minitest/autorun"
require "ucert"

class TestYstTracker < Minitest::Test

  def setup
    @yst_tracker = Ucert::YstTracker.new(:verbose=>false)
  end

  def test_instance_initialize_success
     refute_nil @yst_tracker.yst_user_report
     refute_nil @yst_tracker.yst_org_report
     refute_nil @yst_tracker.file_user_map
     refute_nil @yst_tracker.yst_2_ad_user
     refute_nil @yst_tracker.yst_user_entitlement
     refute_nil @yst_tracker.yst_org_entitlement
  end

  def test_parse_yst_user_report_method
    @test_user_entitlement=@yst_tracker.send(:parse_yst_user_report, File.dirname(__FILE__)+"/yst_testfiles/YiShiTong_User_Test.csv")
    line_cnt=0
    File.open(File.dirname(__FILE__)+"/yst_testfiles/YiShiTong_User_Test.csv",'r:gb18030:UTF-8').each do |line|
      if line_cnt==0
        @title=line.chomp.split(",")
      else
        user=line.chomp.split(",").map {|x| x.gsub(/(\s|\")/,"")}
        assert_equal @title.size, @test_user_entitlement.values[line_cnt-1].size
        assert_equal user, @test_user_entitlement.values[line_cnt-1].values
      end
      line_cnt+=1
    end
    assert_equal line_cnt-1, @test_user_entitlement.size
  end

  def test_parse_yst_org_report_method
    @test_org_entitlement=@yst_tracker.send(:parse_yst_org_report,File.dirname(__FILE__)+"/yst_testfiles/YiShiTong_Org_Test.csv")
    line_cnt=0
    f=File.open( File.dirname(__FILE__)+"/yst_testfiles/YiShiTong_Org_Test.csv",'r:gb18030:UTF-8')
    f.each do |line|
      if line_cnt==0
        @title=line.chomp.split(",")
      else
        org=line.chomp.split(",").map {|x| x.gsub(/(\s|\")/,"")}
        assert_equal @title.size, @test_org_entitlement.values[line_cnt-1].size
        assert_equal org, @test_org_entitlement.values[line_cnt-1].values
      end
      line_cnt+=1
    end
    assert_equal line_cnt-1, @test_org_entitlement.size
  end

    def test_insert_dn
      @yst_tracker.yst_user_entitlement.each do |index, user|
        dn_regex=/\ACN=.+\,OU=.+\,DC=ny,DC=cmbchina,DC=com\z/
        assert_match dn_regex, user["DN"] unless user["DN"].nil?
      end
    end

    def test_save_yst_user_map!
      @yst_tracker.send(:save!, File.dirname(__FILE__)+"/yst_testfiles/yst_user_map_test.txt")
      line_count=0
      File.open(File.dirname(__FILE__)+"/yst_testfiles/yst_user_map_test.txt", 'r').each_line do |line|
        if line_count==0
          assert_match (/\# local Yst to AD user map file created by the .+ class .+ method at: .+/), line
        else
          user_name=line.split(":")[0]
          email=line.split(":")[1].split("|")[0]
          dn=line.split(":")[1].split("|")[1].nil? ? dn="" : dn=line.split(":")[1].split("|")[1].strip
          assert_equal @yst_tracker.yst_user_entitlement[line_count]["USER_NAME"].upcase, user_name
          assert_equal @yst_tracker.yst_user_entitlement[line_count]["EMAIL"].upcase, email
          if dn.empty?
            assert_nil @yst_tracker.yst_user_entitlement[line_count]["DN"]
          else
            assert_equal @yst_tracker.yst_user_entitlement[line_count]["DN"], dn
          end
        end
        line_count+=1
      end
      File.delete(File.dirname(__FILE__)+"/yst_testfiles/yst_user_map_test.txt")
    end

    def test_yst_search_by_dn
      index=0
      File.open(@yst_tracker.file_user_map, 'r').each_line do |line|
        if index>0
          line.split("|")[1].nil? ? dn="" : dn=line.split("|")[1].strip
          assert_equal dn, @yst_tracker.send(:search_by_dn, dn)['DN'] unless dn.empty?
        end
        index=index+1
      end
    end

    def test_yst_search_by_id
      index=0
      File.open(@yst_tracker.yst_user_report, 'r:gb18030:UTF-8').each_line do |line|
        if index>0
          id=line.split(",")[0].gsub('"', '')
          assert_equal id, @yst_tracker.send(:search_by_id, id)['USER_ID']
        end
        index=index+1
      end
    end

  def test_yst_user_entitlement
    column_num=Array.new
    @yst_tracker.yst_user_entitlement.values.map do |rec|
      column_num.push(rec.each.size)
    end
    assert_equal 1, column_num.uniq!.size
    $stdout = StringIO.new
    @yst_tracker.print_user.each
    items=$stdout.string.split("\n")
    items.each do |line|
      assert_match (/\A.+\|(.*\|){#{column_num[0]-1}}\z/), line
      assert_equal column_num[0], line.count("|")
    end
  end

  def test_print_org_entitlement
    column_num=Array.new
    @yst_tracker.yst_org_entitlement.values.map do |rec|
      column_num.push(rec.each.size)
    end
    assert_equal 1, column_num.uniq!.size
    $stdout = StringIO.new
    @yst_tracker.print_org.each
    items=$stdout.string.split("\n")
    items.each do |line|
     assert_match (/\A.+\|(.*\|){#{column_num[0]-1}}\z/), line
     assert_equal column_num[0], line.count("|")
    end
  end
end
